@@ -4,9 +4,9 @@ import (
 	"bytes"
 	"errors"
 	"io"
-	"log"
 	"os"
 	"reflect"
+	"regexp"
 	"testing"
 	"time"
 
@@ -17,6 +17,7 @@ import (
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/services/meta"
 	"github.com/influxdata/influxdb/tsdb"
+	"github.com/uber-go/zap"
 )
 
 const (
@@ -32,28 +33,35 @@ func TestQueryExecutor_ExecuteQuery_SelectStatement(t *testing.T) {
 	e := DefaultQueryExecutor()
 
 	// The meta client should return a single shard owned by the local node.
-	e.MetaClient.ShardsByTimeRangeFn = func(sources influxql.Sources, tmin, tmax time.Time) (a []meta.ShardInfo, err error) {
-		return []meta.ShardInfo{{ID: 100, Owners: []meta.ShardOwner{{NodeID: 0}}}}, nil
+	e.MetaClient.ShardGroupsByTimeRangeFn = func(database, policy string, min, max time.Time) (a []meta.ShardGroupInfo, err error) {
+		return []meta.ShardGroupInfo{
+			{ID: 1, Shards: []meta.ShardInfo{
+				{ID: 100, Owners: []meta.ShardOwner{{NodeID: 0}}},
+			}},
+		}, nil
 	}
 
 	// The TSDB store should return an IteratorCreator for shard.
 	// This IteratorCreator returns a single iterator with "value" in the aux fields.
-	e.TSDBStore.ShardIteratorCreatorFn = func(id uint64) influxql.IteratorCreator {
-		if id != 100 {
-			t.Fatalf("unexpected shard id: %d", id)
+	e.TSDBStore.ShardGroupFn = func(ids []uint64) tsdb.ShardGroup {
+		if !reflect.DeepEqual(ids, []uint64{100}) {
+			t.Fatalf("unexpected shard ids: %v", ids)
 		}
 
-		var ic IteratorCreator
-		ic.CreateIteratorFn = func(opt influxql.IteratorOptions) (influxql.Iterator, error) {
+		var sh MockShard
+		sh.CreateIteratorFn = func(m string, opt influxql.IteratorOptions) (influxql.Iterator, error) {
 			return &FloatIterator{Points: []influxql.FloatPoint{
 				{Name: "cpu", Time: int64(0 * time.Second), Aux: []interface{}{float64(100)}},
 				{Name: "cpu", Time: int64(1 * time.Second), Aux: []interface{}{float64(200)}},
 			}}, nil
 		}
-		ic.FieldDimensionsFn = func(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
+		sh.FieldDimensionsFn = func(measurements []string) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
+			if !reflect.DeepEqual(measurements, []string{"cpu"}) {
+				t.Fatalf("unexpected source: %#v", measurements)
+			}
 			return map[string]influxql.DataType{"value": influxql.Float}, nil, nil
 		}
-		return &ic
+		return &sh
 	}
 
 	// Verify all results from the query.
@@ -80,22 +88,33 @@ func TestQueryExecutor_ExecuteQuery_MaxSelectBucketsN(t *testing.T) {
 	e.StatementExecutor.MaxSelectBucketsN = 3
 
 	// The meta client should return a single shards on the local node.
-	e.MetaClient.ShardsByTimeRangeFn = func(sources influxql.Sources, tmin, tmax time.Time) (a []meta.ShardInfo, err error) {
-		return []meta.ShardInfo{
-			{ID: 100, Owners: []meta.ShardOwner{{NodeID: 0}}},
+	e.MetaClient.ShardGroupsByTimeRangeFn = func(database, policy string, min, max time.Time) (a []meta.ShardGroupInfo, err error) {
+		return []meta.ShardGroupInfo{
+			{ID: 1, Shards: []meta.ShardInfo{
+				{ID: 100, Owners: []meta.ShardOwner{{NodeID: 0}}},
+			}},
 		}, nil
 	}
 
-	var ic IteratorCreator
-	ic.CreateIteratorFn = func(opt influxql.IteratorOptions) (influxql.Iterator, error) {
-		return &FloatIterator{
-			Points: []influxql.FloatPoint{{Name: "cpu", Time: int64(0 * time.Second), Aux: []interface{}{float64(100)}}},
-		}, nil
+	e.TSDBStore.ShardGroupFn = func(ids []uint64) tsdb.ShardGroup {
+		if !reflect.DeepEqual(ids, []uint64{100}) {
+			t.Fatalf("unexpected shard ids: %v", ids)
+		}
+
+		var sh MockShard
+		sh.CreateIteratorFn = func(m string, opt influxql.IteratorOptions) (influxql.Iterator, error) {
+			return &FloatIterator{
+				Points: []influxql.FloatPoint{{Name: "cpu", Time: int64(0 * time.Second), Aux: []interface{}{float64(100)}}},
+			}, nil
+		}
+		sh.FieldDimensionsFn = func(measurements []string) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
+			if !reflect.DeepEqual(measurements, []string{"cpu"}) {
+				t.Fatalf("unexpected source: %#v", measurements)
+			}
+			return map[string]influxql.DataType{"value": influxql.Float}, nil, nil
+		}
+		return &sh
 	}
-	ic.FieldDimensionsFn = func(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
-		return map[string]influxql.DataType{"value": influxql.Float}, nil, nil
-	}
-	e.TSDBStore.ShardIteratorCreatorFn = func(id uint64) influxql.IteratorCreator { return &ic }
 
 	// Verify all results from the query.
 	if a := ReadAllResults(e.ExecuteQuery(`SELECT count(value) FROM cpu WHERE time >= '2000-01-01T00:00:05Z' AND time < '2000-01-01T00:00:35Z' GROUP BY time(10s)`, "db0", 0)); !reflect.DeepEqual(a, []*influxql.Result{
@@ -174,6 +193,69 @@ func TestStatementExecutor_NormalizeDeleteSeries(t *testing.T) {
 	}
 }
 
+type mockAuthorizer struct {
+	AuthorizeDatabaseFn func(influxql.Privilege, string) bool
+}
+
+func (a *mockAuthorizer) AuthorizeDatabase(p influxql.Privilege, name string) bool {
+	return a.AuthorizeDatabaseFn(p, name)
+}
+
+func (m *mockAuthorizer) AuthorizeQuery(database string, query *influxql.Query) error {
+	panic("fail")
+}
+
+func (m *mockAuthorizer) AuthorizeSeriesRead(database string, measurement []byte, tags models.Tags) bool {
+	panic("fail")
+}
+
+func (m *mockAuthorizer) AuthorizeSeriesWrite(database string, measurement []byte, tags models.Tags) bool {
+	panic("fail")
+}
+
+func TestQueryExecutor_ExecuteQuery_ShowDatabases(t *testing.T) {
+	qe := influxql.NewQueryExecutor()
+	qe.StatementExecutor = &coordinator.StatementExecutor{
+		MetaClient: &internal.MetaClientMock{
+			DatabasesFn: func() []meta.DatabaseInfo {
+				return []meta.DatabaseInfo{
+					{Name: "db1"}, {Name: "db2"}, {Name: "db3"}, {Name: "db4"},
+				}
+			},
+		},
+	}
+
+	opt := influxql.ExecutionOptions{
+		Authorizer: &mockAuthorizer{
+			AuthorizeDatabaseFn: func(p influxql.Privilege, name string) bool {
+				return name == "db2" || name == "db4"
+			},
+		},
+	}
+
+	q, err := influxql.ParseQuery("SHOW DATABASES")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	results := ReadAllResults(qe.ExecuteQuery(q, opt, make(chan struct{})))
+	exp := []*influxql.Result{
+		{
+			StatementID: 0,
+			Series: []*models.Row{{
+				Name:    "databases",
+				Columns: []string{"name"},
+				Values: [][]interface{}{
+					{"db2"}, {"db4"},
+				},
+			}},
+		},
+	}
+	if !reflect.DeepEqual(results, exp) {
+		t.Fatalf("unexpected results: exp %s, got %s", spew.Sdump(exp), spew.Sdump(results))
+	}
+}
+
 // QueryExecutor is a test wrapper for coordinator.QueryExecutor.
 type QueryExecutor struct {
 	*influxql.QueryExecutor
@@ -193,6 +275,10 @@ func NewQueryExecutor() *QueryExecutor {
 	e.StatementExecutor = &coordinator.StatementExecutor{
 		MetaClient: &e.MetaClient,
 		TSDBStore:  &e.TSDBStore,
+		ShardMapper: &coordinator.LocalShardMapper{
+			MetaClient: &e.MetaClient,
+			TSDBStore:  &e.TSDBStore,
+		},
 	}
 	e.QueryExecutor.StatementExecutor = e.StatementExecutor
 
@@ -200,7 +286,10 @@ func NewQueryExecutor() *QueryExecutor {
 	if testing.Verbose() {
 		out = io.MultiWriter(out, os.Stderr)
 	}
-	e.QueryExecutor.Logger = log.New(out, "[query] ", log.LstdFlags)
+	e.QueryExecutor.WithLogger(zap.New(
+		zap.NewTextEncoder(),
+		zap.Output(zap.AddSync(out)),
+	))
 
 	return e
 }
@@ -233,8 +322,7 @@ type TSDBStore struct {
 	DeleteRetentionPolicyFn func(database, name string) error
 	DeleteShardFn           func(id uint64) error
 	DeleteSeriesFn          func(database string, sources []influxql.Source, condition influxql.Expr) error
-	DatabaseIndexFn         func(name string) *tsdb.DatabaseIndex
-	ShardIteratorCreatorFn  func(id uint64) influxql.IteratorCreator
+	ShardGroupFn            func(ids []uint64) tsdb.ShardGroup
 }
 
 func (s *TSDBStore) CreateShard(database, policy string, shardID uint64, enabled bool) error {
@@ -276,43 +364,65 @@ func (s *TSDBStore) DeleteSeries(database string, sources []influxql.Source, con
 	return s.DeleteSeriesFn(database, sources, condition)
 }
 
-func (s *TSDBStore) IteratorCreator(shards []meta.ShardInfo, opt *influxql.SelectOptions) (influxql.IteratorCreator, error) {
-	// Generate iterators for each node.
-	ics := make([]influxql.IteratorCreator, 0)
-	if err := func() error {
-		for _, shard := range shards {
-			ic := s.ShardIteratorCreator(shard.ID)
-			if ic == nil {
-				continue
-			}
-			ics = append(ics, ic)
-		}
-
-		return nil
-	}(); err != nil {
-		influxql.IteratorCreators(ics).Close()
-		return nil, err
-	}
-
-	return influxql.IteratorCreators(ics), nil
-}
-
-func (s *TSDBStore) ShardIteratorCreator(id uint64) influxql.IteratorCreator {
-	return s.ShardIteratorCreatorFn(id)
-}
-
-func (s *TSDBStore) DatabaseIndex(name string) *tsdb.DatabaseIndex {
-	return s.DatabaseIndexFn(name)
+func (s *TSDBStore) ShardGroup(ids []uint64) tsdb.ShardGroup {
+	return s.ShardGroupFn(ids)
 }
 
 func (s *TSDBStore) Measurements(database string, cond influxql.Expr) ([]string, error) {
 	return nil, nil
 }
 
+func (s *TSDBStore) MeasurementNames(database string, cond influxql.Expr) ([][]byte, error) {
+	return nil, nil
+}
+
 func (s *TSDBStore) TagValues(database string, cond influxql.Expr) ([]tsdb.TagValues, error) {
 	return nil, nil
 }
 
+type MockShard struct {
+	Measurements      []string
+	FieldDimensionsFn func(measurements []string) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error)
+	CreateIteratorFn  func(m string, opt influxql.IteratorOptions) (influxql.Iterator, error)
+	ExpandSourcesFn   func(sources influxql.Sources) (influxql.Sources, error)
+}
+
+func (sh *MockShard) MeasurementsByRegex(re *regexp.Regexp) []string {
+	names := make([]string, 0, len(sh.Measurements))
+	for _, name := range sh.Measurements {
+		if re.MatchString(name) {
+			names = append(names, name)
+		}
+	}
+	return names
+}
+
+func (sh *MockShard) FieldDimensions(measurements []string) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
+	return sh.FieldDimensionsFn(measurements)
+}
+
+func (sh *MockShard) MapType(measurement, field string) influxql.DataType {
+	f, d, err := sh.FieldDimensions([]string{measurement})
+	if err != nil {
+		return influxql.Unknown
+	}
+
+	if typ, ok := f[field]; ok {
+		return typ
+	} else if _, ok := d[field]; ok {
+		return influxql.Tag
+	}
+	return influxql.Unknown
+}
+
+func (sh *MockShard) CreateIterator(measurement string, opt influxql.IteratorOptions) (influxql.Iterator, error) {
+	return sh.CreateIteratorFn(measurement, opt)
+}
+
+func (sh *MockShard) ExpandSources(sources influxql.Sources) (influxql.Sources, error) {
+	return sh.ExpandSourcesFn(sources)
+}
+
 // MustParseQuery parses s into a query. Panic on error.
 func MustParseQuery(s string) *influxql.Query {
 	q, err := influxql.ParseQuery(s)
@@ -331,25 +441,6 @@ func ReadAllResults(c <-chan *influxql.Result) []*influxql.Result {
 	return a
 }
 
-// IteratorCreator is a mockable implementation of IteratorCreator.
-type IteratorCreator struct {
-	CreateIteratorFn  func(opt influxql.IteratorOptions) (influxql.Iterator, error)
-	FieldDimensionsFn func(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error)
-	ExpandSourcesFn   func(sources influxql.Sources) (influxql.Sources, error)
-}
-
-func (ic *IteratorCreator) CreateIterator(opt influxql.IteratorOptions) (influxql.Iterator, error) {
-	return ic.CreateIteratorFn(opt)
-}
-
-func (ic *IteratorCreator) FieldDimensions(sources influxql.Sources) (fields map[string]influxql.DataType, dimensions map[string]struct{}, err error) {
-	return ic.FieldDimensionsFn(sources)
-}
-
-func (ic *IteratorCreator) ExpandSources(sources influxql.Sources) (influxql.Sources, error) {
-	return ic.ExpandSourcesFn(sources)
-}
-
 // FloatIterator is a represents an iterator that reads from a slice.
 type FloatIterator struct {
 	Points []influxql.FloatPoint
