@@ -1,8 +1,6 @@
 package influxql
 
 import (
-	"bytes"
-	"container/heap"
 	"fmt"
 	"math"
 	"sort"
@@ -67,25 +65,25 @@ func newCountIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatFuncIntegerReducer(FloatCountReduce, &IntegerPoint{Value: 0, Time: ZeroTime})
 			return fn, fn
 		}
-		return &floatReduceIntegerIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceIntegerIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerFuncReducer(IntegerCountReduce, &IntegerPoint{Value: 0, Time: ZeroTime})
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case StringIterator:
 		createFn := func() (StringPointAggregator, IntegerPointEmitter) {
 			fn := NewStringFuncIntegerReducer(StringCountReduce, &IntegerPoint{Value: 0, Time: ZeroTime})
 			return fn, fn
 		}
-		return &stringReduceIntegerIterator{input: newBufStringIterator(input), opt: opt, create: createFn}, nil
+		return newStringReduceIntegerIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, IntegerPointEmitter) {
 			fn := NewBooleanFuncIntegerReducer(BooleanCountReduce, &IntegerPoint{Value: 0, Time: ZeroTime})
 			return fn, fn
 		}
-		return &booleanReduceIntegerIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
+		return newBooleanReduceIntegerIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported count iterator type: %T", input)
 	}
@@ -131,19 +129,19 @@ func newMinIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatFuncReducer(FloatMinReduce, nil)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerFuncReducer(IntegerMinReduce, nil)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
 			fn := NewBooleanFuncReducer(BooleanMinReduce, nil)
 			return fn, fn
 		}
-		return &booleanReduceBooleanIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported min iterator type: %T", input)
 	}
@@ -152,7 +150,7 @@ func newMinIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 // FloatMinReduce returns the minimum value between prev & curr.
 func FloatMinReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 	if prev == nil || curr.Value < prev.Value || (curr.Value == prev.Value && curr.Time < prev.Time) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -160,7 +158,7 @@ func FloatMinReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 // IntegerMinReduce returns the minimum value between prev & curr.
 func IntegerMinReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 	if prev == nil || curr.Value < prev.Value || (curr.Value == prev.Value && curr.Time < prev.Time) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -168,7 +166,7 @@ func IntegerMinReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 // BooleanMinReduce returns the minimum value between prev & curr.
 func BooleanMinReduce(prev, curr *BooleanPoint) (int64, bool, []interface{}) {
 	if prev == nil || (curr.Value != prev.Value && !curr.Value) || (curr.Value == prev.Value && curr.Time < prev.Time) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -181,19 +179,19 @@ func newMaxIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatFuncReducer(FloatMaxReduce, nil)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerFuncReducer(IntegerMaxReduce, nil)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
 			fn := NewBooleanFuncReducer(BooleanMaxReduce, nil)
 			return fn, fn
 		}
-		return &booleanReduceBooleanIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported max iterator type: %T", input)
 	}
@@ -202,7 +200,7 @@ func newMaxIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 // FloatMaxReduce returns the maximum value between prev & curr.
 func FloatMaxReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 	if prev == nil || curr.Value > prev.Value || (curr.Value == prev.Value && curr.Time < prev.Time) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -210,7 +208,7 @@ func FloatMaxReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 // IntegerMaxReduce returns the maximum value between prev & curr.
 func IntegerMaxReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 	if prev == nil || curr.Value > prev.Value || (curr.Value == prev.Value && curr.Time < prev.Time) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -218,7 +216,7 @@ func IntegerMaxReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 // BooleanMaxReduce returns the minimum value between prev & curr.
 func BooleanMaxReduce(prev, curr *BooleanPoint) (int64, bool, []interface{}) {
 	if prev == nil || (curr.Value != prev.Value && curr.Value) || (curr.Value == prev.Value && curr.Time < prev.Time) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -231,13 +229,13 @@ func newSumIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatFuncReducer(FloatSumReduce, &FloatPoint{Value: 0, Time: ZeroTime})
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerFuncReducer(IntegerSumReduce, &IntegerPoint{Value: 0, Time: ZeroTime})
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported sum iterator type: %T", input)
 	}
@@ -267,25 +265,25 @@ func newFirstIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatFuncReducer(FloatFirstReduce, nil)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerFuncReducer(IntegerFirstReduce, nil)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case StringIterator:
 		createFn := func() (StringPointAggregator, StringPointEmitter) {
 			fn := NewStringFuncReducer(StringFirstReduce, nil)
 			return fn, fn
 		}
-		return &stringReduceStringIterator{input: newBufStringIterator(input), opt: opt, create: createFn}, nil
+		return newStringReduceStringIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
 			fn := NewBooleanFuncReducer(BooleanFirstReduce, nil)
 			return fn, fn
 		}
-		return &booleanReduceBooleanIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported first iterator type: %T", input)
 	}
@@ -294,7 +292,7 @@ func newFirstIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 // FloatFirstReduce returns the first point sorted by time.
 func FloatFirstReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 	if prev == nil || curr.Time < prev.Time || (curr.Time == prev.Time && curr.Value > prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -302,7 +300,7 @@ func FloatFirstReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 // IntegerFirstReduce returns the first point sorted by time.
 func IntegerFirstReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 	if prev == nil || curr.Time < prev.Time || (curr.Time == prev.Time && curr.Value > prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -310,7 +308,7 @@ func IntegerFirstReduce(prev, curr *IntegerPoint) (int64, int64, []interface{})
 // StringFirstReduce returns the first point sorted by time.
 func StringFirstReduce(prev, curr *StringPoint) (int64, string, []interface{}) {
 	if prev == nil || curr.Time < prev.Time || (curr.Time == prev.Time && curr.Value > prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -318,7 +316,7 @@ func StringFirstReduce(prev, curr *StringPoint) (int64, string, []interface{}) {
 // BooleanFirstReduce returns the first point sorted by time.
 func BooleanFirstReduce(prev, curr *BooleanPoint) (int64, bool, []interface{}) {
 	if prev == nil || curr.Time < prev.Time || (curr.Time == prev.Time && !curr.Value && prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -331,25 +329,25 @@ func newLastIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatFuncReducer(FloatLastReduce, nil)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerFuncReducer(IntegerLastReduce, nil)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case StringIterator:
 		createFn := func() (StringPointAggregator, StringPointEmitter) {
 			fn := NewStringFuncReducer(StringLastReduce, nil)
 			return fn, fn
 		}
-		return &stringReduceStringIterator{input: newBufStringIterator(input), opt: opt, create: createFn}, nil
+		return newStringReduceStringIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
 			fn := NewBooleanFuncReducer(BooleanLastReduce, nil)
 			return fn, fn
 		}
-		return &booleanReduceBooleanIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported last iterator type: %T", input)
 	}
@@ -358,7 +356,7 @@ func newLastIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 // FloatLastReduce returns the last point sorted by time.
 func FloatLastReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 	if prev == nil || curr.Time > prev.Time || (curr.Time == prev.Time && curr.Value > prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -366,7 +364,7 @@ func FloatLastReduce(prev, curr *FloatPoint) (int64, float64, []interface{}) {
 // IntegerLastReduce returns the last point sorted by time.
 func IntegerLastReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 	if prev == nil || curr.Time > prev.Time || (curr.Time == prev.Time && curr.Value > prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -374,7 +372,7 @@ func IntegerLastReduce(prev, curr *IntegerPoint) (int64, int64, []interface{}) {
 // StringLastReduce returns the first point sorted by time.
 func StringLastReduce(prev, curr *StringPoint) (int64, string, []interface{}) {
 	if prev == nil || curr.Time > prev.Time || (curr.Time == prev.Time && curr.Value > prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -382,7 +380,7 @@ func StringLastReduce(prev, curr *StringPoint) (int64, string, []interface{}) {
 // BooleanLastReduce returns the first point sorted by time.
 func BooleanLastReduce(prev, curr *BooleanPoint) (int64, bool, []interface{}) {
 	if prev == nil || curr.Time > prev.Time || (curr.Time == prev.Time && curr.Value && !prev.Value) {
-		return curr.Time, curr.Value, curr.Aux
+		return curr.Time, curr.Value, cloneAux(curr.Aux)
 	}
 	return prev.Time, prev.Value, prev.Aux
 }
@@ -395,25 +393,25 @@ func NewDistinctIterator(input Iterator, opt IteratorOptions) (Iterator, error)
 			fn := NewFloatDistinctReducer()
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerDistinctReducer()
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case StringIterator:
 		createFn := func() (StringPointAggregator, StringPointEmitter) {
 			fn := NewStringDistinctReducer()
 			return fn, fn
 		}
-		return &stringReduceStringIterator{input: newBufStringIterator(input), opt: opt, create: createFn}, nil
+		return newStringReduceStringIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
 			fn := NewBooleanDistinctReducer()
 			return fn, fn
 		}
-		return &booleanReduceBooleanIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported distinct iterator type: %T", input)
 	}
@@ -427,13 +425,13 @@ func newMeanIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatMeanReducer()
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, FloatPointEmitter) {
 			fn := NewIntegerMeanReducer()
 			return fn, fn
 		}
-		return &integerReduceFloatIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceFloatIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported mean iterator type: %T", input)
 	}
@@ -452,13 +450,13 @@ func newMedianIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatSliceFuncReducer(FloatMedianReduceSlice)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, FloatPointEmitter) {
 			fn := NewIntegerSliceFuncFloatReducer(IntegerMedianReduceSlice)
 			return fn, fn
 		}
-		return &integerReduceFloatIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceFloatIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported median iterator type: %T", input)
 	}
@@ -502,33 +500,31 @@ func IntegerMedianReduceSlice(a []IntegerPoint) []FloatPoint {
 
 // newModeIterator returns an iterator for operating on a mode() call.
 func NewModeIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
-
 	switch input := input.(type) {
 	case FloatIterator:
 		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
 			fn := NewFloatSliceFuncReducer(FloatModeReduceSlice)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerSliceFuncReducer(IntegerModeReduceSlice)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	case StringIterator:
 		createFn := func() (StringPointAggregator, StringPointEmitter) {
 			fn := NewStringSliceFuncReducer(StringModeReduceSlice)
 			return fn, fn
 		}
-		return &stringReduceStringIterator{input: newBufStringIterator(input), opt: opt, create: createFn}, nil
+		return newStringReduceStringIterator(input, opt, createFn), nil
 	case BooleanIterator:
 		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
 			fn := NewBooleanSliceFuncReducer(BooleanModeReduceSlice)
 			return fn, fn
 		}
-		return &booleanReduceBooleanIterator{input: newBufBooleanIterator(input), opt: opt, create: createFn}, nil
-
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported median iterator type: %T", input)
 	}
@@ -540,7 +536,6 @@ func FloatModeReduceSlice(a []FloatPoint) []FloatPoint {
 		return a
 	}
 
-	// fmt.Println(a[0])
 	sort.Sort(floatPointsByValue(a))
 
 	mostFreq := 0
@@ -670,19 +665,13 @@ func newStddevIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatSliceFuncReducer(FloatStddevReduceSlice)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, FloatPointEmitter) {
 			fn := NewIntegerSliceFuncFloatReducer(IntegerStddevReduceSlice)
 			return fn, fn
 		}
-		return &integerReduceFloatIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
-	case StringIterator:
-		createFn := func() (StringPointAggregator, StringPointEmitter) {
-			fn := NewStringSliceFuncReducer(StringStddevReduceSlice)
-			return fn, fn
-		}
-		return &stringReduceStringIterator{input: newBufStringIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceFloatIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported stddev iterator type: %T", input)
 	}
@@ -746,11 +735,6 @@ func IntegerStddevReduceSlice(a []IntegerPoint) []FloatPoint {
 	}}
 }
 
-// StringStddevReduceSlice always returns "".
-func StringStddevReduceSlice(a []StringPoint) []StringPoint {
-	return []StringPoint{{Time: ZeroTime, Value: ""}}
-}
-
 // newSpreadIterator returns an iterator for operating on a spread() call.
 func newSpreadIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 	switch input := input.(type) {
@@ -759,13 +743,13 @@ func newSpreadIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
 			fn := NewFloatSliceFuncReducer(FloatSpreadReduceSlice)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerSliceFuncReducer(IntegerSpreadReduceSlice)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported spread iterator type: %T", input)
 	}
@@ -797,292 +781,52 @@ func IntegerSpreadReduceSlice(a []IntegerPoint) []IntegerPoint {
 	return []IntegerPoint{{Time: ZeroTime, Value: max - min}}
 }
 
-func newTopIterator(input Iterator, opt IteratorOptions, n *IntegerLiteral, tags []int) (Iterator, error) {
+func newTopIterator(input Iterator, opt IteratorOptions, n int, keepTags bool) (Iterator, error) {
 	switch input := input.(type) {
 	case FloatIterator:
-		aggregateFn := NewFloatTopReduceSliceFunc(int(n.Val), tags, opt.Interval)
 		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
-			fn := NewFloatSliceFuncReducer(aggregateFn)
+			fn := NewFloatTopReducer(n)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		itr := newFloatReduceFloatIterator(input, opt, createFn)
+		itr.keepTags = keepTags
+		return itr, nil
 	case IntegerIterator:
-		aggregateFn := NewIntegerTopReduceSliceFunc(int(n.Val), tags, opt.Interval)
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
-			fn := NewIntegerSliceFuncReducer(aggregateFn)
+			fn := NewIntegerTopReducer(n)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		itr := newIntegerReduceIntegerIterator(input, opt, createFn)
+		itr.keepTags = keepTags
+		return itr, nil
 	default:
 		return nil, fmt.Errorf("unsupported top iterator type: %T", input)
 	}
 }
 
-// NewFloatTopReduceSliceFunc returns the top values within a window.
-func NewFloatTopReduceSliceFunc(n int, tags []int, interval Interval) FloatReduceSliceFunc {
-	return func(a []FloatPoint) []FloatPoint {
-		// Filter by tags if they exist.
-		if len(tags) > 0 {
-			a = filterFloatByUniqueTags(a, tags, func(cur, p *FloatPoint) bool {
-				return p.Value > cur.Value || (p.Value == cur.Value && p.Time < cur.Time)
-			})
-		}
-
-		// If we ask for more elements than exist, restrict n to be the length of the array.
-		size := n
-		if size > len(a) {
-			size = len(a)
-		}
-
-		// Construct a heap preferring higher values and breaking ties
-		// based on the earliest time for a point.
-		h := floatPointsSortBy(a, func(a, b *FloatPoint) bool {
-			if a.Value != b.Value {
-				return a.Value > b.Value
-			}
-			return a.Time < b.Time
-		})
-		heap.Init(h)
-
-		// Pop the first n elements and then sort by time.
-		points := make([]FloatPoint, 0, size)
-		for i := 0; i < size; i++ {
-			p := heap.Pop(h).(FloatPoint)
-			points = append(points, p)
-		}
-
-		// Either zero out all values or sort the points by time
-		// depending on if a time interval was given or not.
-		if !interval.IsZero() {
-			for i := range points {
-				points[i].Time = ZeroTime
-			}
-		} else {
-			sort.Stable(floatPointsByTime(points))
-		}
-		return points
-	}
-}
-
-// NewIntegerTopReduceSliceFunc returns the top values within a window.
-func NewIntegerTopReduceSliceFunc(n int, tags []int, interval Interval) IntegerReduceSliceFunc {
-	return func(a []IntegerPoint) []IntegerPoint {
-		// Filter by tags if they exist.
-		if len(tags) > 0 {
-			a = filterIntegerByUniqueTags(a, tags, func(cur, p *IntegerPoint) bool {
-				return p.Value > cur.Value || (p.Value == cur.Value && p.Time < cur.Time)
-			})
-		}
-
-		// If we ask for more elements than exist, restrict n to be the length of the array.
-		size := n
-		if size > len(a) {
-			size = len(a)
-		}
-
-		// Construct a heap preferring higher values and breaking ties
-		// based on the earliest time for a point.
-		h := integerPointsSortBy(a, func(a, b *IntegerPoint) bool {
-			if a.Value != b.Value {
-				return a.Value > b.Value
-			}
-			return a.Time < b.Time
-		})
-		heap.Init(h)
-
-		// Pop the first n elements and then sort by time.
-		points := make([]IntegerPoint, 0, size)
-		for i := 0; i < size; i++ {
-			p := heap.Pop(h).(IntegerPoint)
-			points = append(points, p)
-		}
-
-		// Either zero out all values or sort the points by time
-		// depending on if a time interval was given or not.
-		if !interval.IsZero() {
-			for i := range points {
-				points[i].Time = ZeroTime
-			}
-		} else {
-			sort.Stable(integerPointsByTime(points))
-		}
-		return points
-	}
-}
-
-func newBottomIterator(input Iterator, opt IteratorOptions, n *IntegerLiteral, tags []int) (Iterator, error) {
+func newBottomIterator(input Iterator, opt IteratorOptions, n int, keepTags bool) (Iterator, error) {
 	switch input := input.(type) {
 	case FloatIterator:
-		aggregateFn := NewFloatBottomReduceSliceFunc(int(n.Val), tags, opt.Interval)
 		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
-			fn := NewFloatSliceFuncReducer(aggregateFn)
+			fn := NewFloatBottomReducer(n)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		itr := newFloatReduceFloatIterator(input, opt, createFn)
+		itr.keepTags = keepTags
+		return itr, nil
 	case IntegerIterator:
-		aggregateFn := NewIntegerBottomReduceSliceFunc(int(n.Val), tags, opt.Interval)
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
-			fn := NewIntegerSliceFuncReducer(aggregateFn)
+			fn := NewIntegerBottomReducer(n)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		itr := newIntegerReduceIntegerIterator(input, opt, createFn)
+		itr.keepTags = keepTags
+		return itr, nil
 	default:
 		return nil, fmt.Errorf("unsupported bottom iterator type: %T", input)
 	}
 }
 
-// NewFloatBottomReduceSliceFunc returns the bottom values within a window.
-func NewFloatBottomReduceSliceFunc(n int, tags []int, interval Interval) FloatReduceSliceFunc {
-	return func(a []FloatPoint) []FloatPoint {
-		// Filter by tags if they exist.
-		if len(tags) > 0 {
-			a = filterFloatByUniqueTags(a, tags, func(cur, p *FloatPoint) bool {
-				return p.Value < cur.Value || (p.Value == cur.Value && p.Time < cur.Time)
-			})
-		}
-
-		// If we ask for more elements than exist, restrict n to be the length of the array.
-		size := n
-		if size > len(a) {
-			size = len(a)
-		}
-
-		// Construct a heap preferring lower values and breaking ties
-		// based on the earliest time for a point.
-		h := floatPointsSortBy(a, func(a, b *FloatPoint) bool {
-			if a.Value != b.Value {
-				return a.Value < b.Value
-			}
-			return a.Time < b.Time
-		})
-		heap.Init(h)
-
-		// Pop the first n elements and then sort by time.
-		points := make([]FloatPoint, 0, size)
-		for i := 0; i < size; i++ {
-			p := heap.Pop(h).(FloatPoint)
-			points = append(points, p)
-		}
-
-		// Either zero out all values or sort the points by time
-		// depending on if a time interval was given or not.
-		if !interval.IsZero() {
-			for i := range points {
-				points[i].Time = ZeroTime
-			}
-		} else {
-			sort.Stable(floatPointsByTime(points))
-		}
-		return points
-	}
-}
-
-// NewIntegerBottomReduceSliceFunc returns the bottom values within a window.
-func NewIntegerBottomReduceSliceFunc(n int, tags []int, interval Interval) IntegerReduceSliceFunc {
-	return func(a []IntegerPoint) []IntegerPoint {
-		// Filter by tags if they exist.
-		if len(tags) > 0 {
-			a = filterIntegerByUniqueTags(a, tags, func(cur, p *IntegerPoint) bool {
-				return p.Value < cur.Value || (p.Value == cur.Value && p.Time < cur.Time)
-			})
-		}
-
-		// If we ask for more elements than exist, restrict n to be the length of the array.
-		size := n
-		if size > len(a) {
-			size = len(a)
-		}
-
-		// Construct a heap preferring lower values and breaking ties
-		// based on the earliest time for a point.
-		h := integerPointsSortBy(a, func(a, b *IntegerPoint) bool {
-			if a.Value != b.Value {
-				return a.Value < b.Value
-			}
-			return a.Time < b.Time
-		})
-		heap.Init(h)
-
-		// Pop the first n elements and then sort by time.
-		points := make([]IntegerPoint, 0, size)
-		for i := 0; i < size; i++ {
-			p := heap.Pop(h).(IntegerPoint)
-			points = append(points, p)
-		}
-
-		// Either zero out all values or sort the points by time
-		// depending on if a time interval was given or not.
-		if !interval.IsZero() {
-			for i := range points {
-				points[i].Time = ZeroTime
-			}
-		} else {
-			sort.Stable(integerPointsByTime(points))
-		}
-		return points
-	}
-}
-
-func filterFloatByUniqueTags(a []FloatPoint, tags []int, cmpFunc func(cur, p *FloatPoint) bool) []FloatPoint {
-	pointMap := make(map[string]FloatPoint)
-	for _, p := range a {
-		keyBuf := bytes.NewBuffer(nil)
-		for i, index := range tags {
-			if i > 0 {
-				keyBuf.WriteString(",")
-			}
-			fmt.Fprintf(keyBuf, "%s", p.Aux[index])
-		}
-		key := keyBuf.String()
-
-		cur, ok := pointMap[key]
-		if ok {
-			if cmpFunc(&cur, &p) {
-				pointMap[key] = p
-			}
-		} else {
-			pointMap[key] = p
-		}
-	}
-
-	// Recreate the original array with our new filtered list.
-	points := make([]FloatPoint, 0, len(pointMap))
-	for _, p := range pointMap {
-		points = append(points, p)
-	}
-	return points
-}
-
-func filterIntegerByUniqueTags(a []IntegerPoint, tags []int, cmpFunc func(cur, p *IntegerPoint) bool) []IntegerPoint {
-	pointMap := make(map[string]IntegerPoint)
-	for _, p := range a {
-		keyBuf := bytes.NewBuffer(nil)
-		for i, index := range tags {
-			if i > 0 {
-				keyBuf.WriteString(",")
-			}
-			fmt.Fprintf(keyBuf, "%s", p.Aux[index])
-		}
-		key := keyBuf.String()
-
-		cur, ok := pointMap[key]
-		if ok {
-			if cmpFunc(&cur, &p) {
-				pointMap[key] = p
-			}
-		} else {
-			pointMap[key] = p
-		}
-	}
-
-	// Recreate the original array with our new filtered list.
-	points := make([]IntegerPoint, 0, len(pointMap))
-	for _, p := range pointMap {
-		points = append(points, p)
-	}
-	return points
-}
-
 // newPercentileIterator returns an iterator for operating on a percentile() call.
 func newPercentileIterator(input Iterator, opt IteratorOptions, percentile float64) (Iterator, error) {
 	switch input := input.(type) {
@@ -1092,14 +836,14 @@ func newPercentileIterator(input Iterator, opt IteratorOptions, percentile float
 			fn := NewFloatSliceFuncReducer(floatPercentileReduceSlice)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		integerPercentileReduceSlice := NewIntegerPercentileReduceSliceFunc(percentile)
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
 			fn := NewIntegerSliceFuncReducer(integerPercentileReduceSlice)
 			return fn, fn
 		}
-		return &integerReduceIntegerIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported percentile iterator type: %T", input)
 	}
@@ -1116,7 +860,7 @@ func NewFloatPercentileReduceSliceFunc(percentile float64) FloatReduceSliceFunc
 		}
 
 		sort.Sort(floatPointsByValue(a))
-		return []FloatPoint{{Time: a[i].Time, Value: a[i].Value, Aux: a[i].Aux}}
+		return []FloatPoint{{Time: a[i].Time, Value: a[i].Value, Aux: cloneAux(a[i].Aux)}}
 	}
 }
 
@@ -1131,7 +875,7 @@ func NewIntegerPercentileReduceSliceFunc(percentile float64) IntegerReduceSliceF
 		}
 
 		sort.Sort(integerPointsByValue(a))
-		return []IntegerPoint{{Time: ZeroTime, Value: a[i].Value, Aux: a[i].Aux}}
+		return []IntegerPoint{{Time: a[i].Time, Value: a[i].Value, Aux: cloneAux(a[i].Aux)}}
 	}
 }
 
@@ -1156,17 +900,17 @@ func newDerivativeIterator(input Iterator, opt IteratorOptions, interval Interva
 }
 
 // newDifferenceIterator returns an iterator for operating on a difference() call.
-func newDifferenceIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
+func newDifferenceIterator(input Iterator, opt IteratorOptions, isNonNegative bool) (Iterator, error) {
 	switch input := input.(type) {
 	case FloatIterator:
 		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
-			fn := NewFloatDifferenceReducer()
+			fn := NewFloatDifferenceReducer(isNonNegative)
 			return fn, fn
 		}
 		return newFloatStreamFloatIterator(input, createFn, opt), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
-			fn := NewIntegerDifferenceReducer()
+			fn := NewIntegerDifferenceReducer(isNonNegative)
 			return fn, fn
 		}
 		return newIntegerStreamIntegerIterator(input, createFn, opt), nil
@@ -1227,7 +971,27 @@ func newMovingAverageIterator(input Iterator, n int, opt IteratorOptions) (Itera
 	}
 }
 
-// newHoltWintersIterator returns an iterator for operating on a elapsed() call.
+// newCumulativeSumIterator returns an iterator for operating on a cumulative_sum() call.
+func newCumulativeSumIterator(input Iterator, opt IteratorOptions) (Iterator, error) {
+	switch input := input.(type) {
+	case FloatIterator:
+		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
+			fn := NewFloatCumulativeSumReducer()
+			return fn, fn
+		}
+		return newFloatStreamFloatIterator(input, createFn, opt), nil
+	case IntegerIterator:
+		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
+			fn := NewIntegerCumulativeSumReducer()
+			return fn, fn
+		}
+		return newIntegerStreamIntegerIterator(input, createFn, opt), nil
+	default:
+		return nil, fmt.Errorf("unsupported cumulative sum iterator type: %T", input)
+	}
+}
+
+// newHoltWintersIterator returns an iterator for operating on a holt_winters() call.
 func newHoltWintersIterator(input Iterator, opt IteratorOptions, h, m int, includeFitData bool, interval time.Duration) (Iterator, error) {
 	switch input := input.(type) {
 	case FloatIterator:
@@ -1235,14 +999,71 @@ func newHoltWintersIterator(input Iterator, opt IteratorOptions, h, m int, inclu
 			fn := NewFloatHoltWintersReducer(h, m, includeFitData, interval)
 			return fn, fn
 		}
-		return &floatReduceFloatIterator{input: newBufFloatIterator(input), opt: opt, create: createFn}, nil
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
 	case IntegerIterator:
 		createFn := func() (IntegerPointAggregator, FloatPointEmitter) {
 			fn := NewFloatHoltWintersReducer(h, m, includeFitData, interval)
 			return fn, fn
 		}
-		return &integerReduceFloatIterator{input: newBufIntegerIterator(input), opt: opt, create: createFn}, nil
+		return newIntegerReduceFloatIterator(input, opt, createFn), nil
 	default:
 		return nil, fmt.Errorf("unsupported elapsed iterator type: %T", input)
 	}
 }
+
+// NewSampleIterator returns an iterator for operating on a sample() call (exported for use in test).
+func NewSampleIterator(input Iterator, opt IteratorOptions, size int) (Iterator, error) {
+	return newSampleIterator(input, opt, size)
+}
+
+// newSampleIterator returns an iterator for operating on a sample() call.
+func newSampleIterator(input Iterator, opt IteratorOptions, size int) (Iterator, error) {
+	switch input := input.(type) {
+	case FloatIterator:
+		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
+			fn := NewFloatSampleReducer(size)
+			return fn, fn
+		}
+		return newFloatReduceFloatIterator(input, opt, createFn), nil
+	case IntegerIterator:
+		createFn := func() (IntegerPointAggregator, IntegerPointEmitter) {
+			fn := NewIntegerSampleReducer(size)
+			return fn, fn
+		}
+		return newIntegerReduceIntegerIterator(input, opt, createFn), nil
+	case StringIterator:
+		createFn := func() (StringPointAggregator, StringPointEmitter) {
+			fn := NewStringSampleReducer(size)
+			return fn, fn
+		}
+		return newStringReduceStringIterator(input, opt, createFn), nil
+	case BooleanIterator:
+		createFn := func() (BooleanPointAggregator, BooleanPointEmitter) {
+			fn := NewBooleanSampleReducer(size)
+			return fn, fn
+		}
+		return newBooleanReduceBooleanIterator(input, opt, createFn), nil
+	default:
+		return nil, fmt.Errorf("unsupported elapsed iterator type: %T", input)
+	}
+}
+
+// newIntegralIterator returns an iterator for operating on a integral() call.
+func newIntegralIterator(input Iterator, opt IteratorOptions, interval Interval) (Iterator, error) {
+	switch input := input.(type) {
+	case FloatIterator:
+		createFn := func() (FloatPointAggregator, FloatPointEmitter) {
+			fn := NewFloatIntegralReducer(interval, opt)
+			return fn, fn
+		}
+		return newFloatStreamFloatIterator(input, createFn, opt), nil
+	case IntegerIterator:
+		createFn := func() (IntegerPointAggregator, FloatPointEmitter) {
+			fn := NewIntegerIntegralReducer(interval, opt)
+			return fn, fn
+		}
+		return newIntegerStreamFloatIterator(input, createFn, opt), nil
+	default:
+		return nil, fmt.Errorf("unsupported integral iterator type: %T", input)
+	}
+}
