@@ -1,7 +1,9 @@
 package influxql
 
 import (
+	"container/heap"
 	"math"
+	"sort"
 	"time"
 
 	"github.com/influxdata/influxdb/influxql/neldermead"
@@ -185,15 +187,17 @@ func (r *IntegerDerivativeReducer) Emit() []FloatPoint {
 
 // FloatDifferenceReducer calculates the derivative of the aggregated points.
 type FloatDifferenceReducer struct {
-	prev FloatPoint
-	curr FloatPoint
+	isNonNegative bool
+	prev          FloatPoint
+	curr          FloatPoint
 }
 
 // NewFloatDifferenceReducer creates a new FloatDifferenceReducer.
-func NewFloatDifferenceReducer() *FloatDifferenceReducer {
+func NewFloatDifferenceReducer(isNonNegative bool) *FloatDifferenceReducer {
 	return &FloatDifferenceReducer{
-		prev: FloatPoint{Nil: true},
-		curr: FloatPoint{Nil: true},
+		isNonNegative: isNonNegative,
+		prev:          FloatPoint{Nil: true},
+		curr:          FloatPoint{Nil: true},
 	}
 }
 
@@ -216,6 +220,12 @@ func (r *FloatDifferenceReducer) Emit() []FloatPoint {
 		// Calculate the difference of successive points.
 		value := r.curr.Value - r.prev.Value
 
+		// If it is non_negative_difference discard any negative value. Since
+		// prev is still marked as unread. The correctness can be ensured.
+		if r.isNonNegative && value < 0 {
+			return nil
+		}
+
 		// Mark this point as read by changing the previous point to nil.
 		r.prev.Nil = true
 		return []FloatPoint{{Time: r.curr.Time, Value: value}}
@@ -225,15 +235,17 @@ func (r *FloatDifferenceReducer) Emit() []FloatPoint {
 
 // IntegerDifferenceReducer calculates the derivative of the aggregated points.
 type IntegerDifferenceReducer struct {
-	prev IntegerPoint
-	curr IntegerPoint
+	isNonNegative bool
+	prev          IntegerPoint
+	curr          IntegerPoint
 }
 
 // NewIntegerDifferenceReducer creates a new IntegerDifferenceReducer.
-func NewIntegerDifferenceReducer() *IntegerDifferenceReducer {
+func NewIntegerDifferenceReducer(isNonNegative bool) *IntegerDifferenceReducer {
 	return &IntegerDifferenceReducer{
-		prev: IntegerPoint{Nil: true},
-		curr: IntegerPoint{Nil: true},
+		isNonNegative: isNonNegative,
+		prev:          IntegerPoint{Nil: true},
+		curr:          IntegerPoint{Nil: true},
 	}
 }
 
@@ -256,8 +268,15 @@ func (r *IntegerDifferenceReducer) Emit() []IntegerPoint {
 		// Calculate the difference of successive points.
 		value := r.curr.Value - r.prev.Value
 
+		// If it is non_negative_difference discard any negative value. Since
+		// prev is still marked as unread. The correctness can be ensured.
+		if r.isNonNegative && value < 0 {
+			return nil
+		}
+
 		// Mark this point as read by changing the previous point to nil.
 		r.prev.Nil = true
+
 		return []IntegerPoint{{Time: r.curr.Time, Value: value}}
 	}
 	return nil
@@ -657,7 +676,6 @@ func (r *FloatHoltWintersReducer) forecast(h int, params []float64) []float64 {
 
 	lT := params[4]
 	bT := params[5]
-	sT := 0.0
 
 	// seasonals is a ring buffer of past sT values
 	var seasonals []float64
@@ -683,6 +701,7 @@ func (r *FloatHoltWintersReducer) forecast(h int, params []float64) []float64 {
 			stm = seasonals[(t-m+so)%m]
 			stmh = seasonals[(t-m+hm+so)%m]
 		}
+		var sT float64
 		yT, lT, bT, sT = r.next(
 			params[0], // alpha
 			params[1], // beta
@@ -757,3 +776,388 @@ func (r *FloatHoltWintersReducer) constrain(x []float64) {
 		x[3] = 0
 	}
 }
+
+// FloatIntegralReducer calculates the time-integral of the aggregated points.
+type FloatIntegralReducer struct {
+	interval Interval
+	sum      float64
+	prev     FloatPoint
+	window   struct {
+		start int64
+		end   int64
+	}
+	ch  chan FloatPoint
+	opt IteratorOptions
+}
+
+// NewFloatIntegralReducer creates a new FloatIntegralReducer.
+func NewFloatIntegralReducer(interval Interval, opt IteratorOptions) *FloatIntegralReducer {
+	return &FloatIntegralReducer{
+		interval: interval,
+		prev:     FloatPoint{Nil: true},
+		ch:       make(chan FloatPoint, 1),
+		opt:      opt,
+	}
+}
+
+// AggregateFloat aggregates a point into the reducer.
+func (r *FloatIntegralReducer) AggregateFloat(p *FloatPoint) {
+	// If this is the first point, just save it
+	if r.prev.Nil {
+		r.prev = *p
+		if !r.opt.Interval.IsZero() {
+			// Record the end of the time interval.
+			// We do not care for whether the last number is inclusive or exclusive
+			// because we treat both the same for the involved math.
+			if r.opt.Ascending {
+				r.window.start, r.window.end = r.opt.Window(p.Time)
+			} else {
+				r.window.end, r.window.start = r.opt.Window(p.Time)
+			}
+		}
+		return
+	}
+
+	// If this point has the same timestamp as the previous one,
+	// skip the point. Points sent into this reducer are expected
+	// to be fed in order.
+	if r.prev.Time == p.Time {
+		r.prev = *p
+		return
+	} else if !r.opt.Interval.IsZero() && ((r.opt.Ascending && p.Time >= r.window.end) || (!r.opt.Ascending && p.Time <= r.window.end)) {
+		// If our previous time is not equal to the window, we need to
+		// interpolate the area at the end of this interval.
+		if r.prev.Time != r.window.end {
+			value := linearFloat(r.window.end, r.prev.Time, p.Time, r.prev.Value, p.Value)
+			elapsed := float64(r.window.end-r.prev.Time) / float64(r.interval.Duration)
+			r.sum += 0.5 * (value + r.prev.Value) * elapsed
+
+			r.prev.Value = value
+			r.prev.Time = r.window.end
+		}
+
+		// Emit the current point through the channel and then clear it.
+		r.ch <- FloatPoint{Time: r.window.start, Value: r.sum}
+		if r.opt.Ascending {
+			r.window.start, r.window.end = r.opt.Window(p.Time)
+		} else {
+			r.window.end, r.window.start = r.opt.Window(p.Time)
+		}
+		r.sum = 0.0
+	}
+
+	// Normal operation: update the sum using the trapezium rule
+	elapsed := float64(p.Time-r.prev.Time) / float64(r.interval.Duration)
+	r.sum += 0.5 * (p.Value + r.prev.Value) * elapsed
+	r.prev = *p
+}
+
+// Emit emits the time-integral of the aggregated points as a single point.
+// InfluxQL convention dictates that outside a group-by-time clause we return
+// a timestamp of zero.  Within a group-by-time, we can set the time to ZeroTime
+// and a higher level will change it to the start of the time group.
+func (r *FloatIntegralReducer) Emit() []FloatPoint {
+	select {
+	case pt, ok := <-r.ch:
+		if !ok {
+			return nil
+		}
+		return []FloatPoint{pt}
+	default:
+		return nil
+	}
+}
+
+// Close flushes any in progress points to ensure any remaining points are
+// emitted.
+func (r *FloatIntegralReducer) Close() error {
+	// If our last point is at the start time, then discard this point since
+	// there is no area within this bucket. Otherwise, send off what we
+	// currently have as the final point.
+	if !r.prev.Nil && r.prev.Time != r.window.start {
+		r.ch <- FloatPoint{Time: r.window.start, Value: r.sum}
+	}
+	close(r.ch)
+	return nil
+}
+
+// IntegerIntegralReducer calculates the time-integral of the aggregated points.
+type IntegerIntegralReducer struct {
+	interval Interval
+	sum      float64
+	prev     IntegerPoint
+	window   struct {
+		start int64
+		end   int64
+	}
+	ch  chan FloatPoint
+	opt IteratorOptions
+}
+
+// NewIntegerIntegralReducer creates a new IntegerIntegralReducer.
+func NewIntegerIntegralReducer(interval Interval, opt IteratorOptions) *IntegerIntegralReducer {
+	return &IntegerIntegralReducer{
+		interval: interval,
+		prev:     IntegerPoint{Nil: true},
+		ch:       make(chan FloatPoint, 1),
+		opt:      opt,
+	}
+}
+
+// AggregateInteger aggregates a point into the reducer.
+func (r *IntegerIntegralReducer) AggregateInteger(p *IntegerPoint) {
+	// If this is the first point, just save it
+	if r.prev.Nil {
+		r.prev = *p
+
+		// Record the end of the time interval.
+		// We do not care for whether the last number is inclusive or exclusive
+		// because we treat both the same for the involved math.
+		if r.opt.Ascending {
+			r.window.start, r.window.end = r.opt.Window(p.Time)
+		} else {
+			r.window.end, r.window.start = r.opt.Window(p.Time)
+		}
+
+		// If we see the minimum allowable time, set the time to zero so we don't
+		// break the default returned time for aggregate queries without times.
+		if r.window.start == MinTime {
+			r.window.start = 0
+		}
+		return
+	}
+
+	// If this point has the same timestamp as the previous one,
+	// skip the point. Points sent into this reducer are expected
+	// to be fed in order.
+	value := float64(p.Value)
+	if r.prev.Time == p.Time {
+		r.prev = *p
+		return
+	} else if (r.opt.Ascending && p.Time >= r.window.end) || (!r.opt.Ascending && p.Time <= r.window.end) {
+		// If our previous time is not equal to the window, we need to
+		// interpolate the area at the end of this interval.
+		if r.prev.Time != r.window.end {
+			value = linearFloat(r.window.end, r.prev.Time, p.Time, float64(r.prev.Value), value)
+			elapsed := float64(r.window.end-r.prev.Time) / float64(r.interval.Duration)
+			r.sum += 0.5 * (value + float64(r.prev.Value)) * elapsed
+
+			r.prev.Time = r.window.end
+		}
+
+		// Emit the current point through the channel and then clear it.
+		r.ch <- FloatPoint{Time: r.window.start, Value: r.sum}
+		if r.opt.Ascending {
+			r.window.start, r.window.end = r.opt.Window(p.Time)
+		} else {
+			r.window.end, r.window.start = r.opt.Window(p.Time)
+		}
+		r.sum = 0.0
+	}
+
+	// Normal operation: update the sum using the trapezium rule
+	elapsed := float64(p.Time-r.prev.Time) / float64(r.interval.Duration)
+	r.sum += 0.5 * (value + float64(r.prev.Value)) * elapsed
+	r.prev = *p
+}
+
+// Emit emits the time-integral of the aggregated points as a single FLOAT point
+// InfluxQL convention dictates that outside a group-by-time clause we return
+// a timestamp of zero.  Within a group-by-time, we can set the time to ZeroTime
+// and a higher level will change it to the start of the time group.
+func (r *IntegerIntegralReducer) Emit() []FloatPoint {
+	select {
+	case pt, ok := <-r.ch:
+		if !ok {
+			return nil
+		}
+		return []FloatPoint{pt}
+	default:
+		return nil
+	}
+}
+
+// Close flushes any in progress points to ensure any remaining points are
+// emitted.
+func (r *IntegerIntegralReducer) Close() error {
+	// If our last point is at the start time, then discard this point since
+	// there is no area within this bucket. Otherwise, send off what we
+	// currently have as the final point.
+	if !r.prev.Nil && r.prev.Time != r.window.start {
+		r.ch <- FloatPoint{Time: r.window.start, Value: r.sum}
+	}
+	close(r.ch)
+	return nil
+}
+
+type FloatTopReducer struct {
+	h *floatPointsByFunc
+}
+
+func NewFloatTopReducer(n int) *FloatTopReducer {
+	return &FloatTopReducer{
+		h: floatPointsSortBy(make([]FloatPoint, 0, n), func(a, b *FloatPoint) bool {
+			if a.Value != b.Value {
+				return a.Value < b.Value
+			}
+			return a.Time > b.Time
+		}),
+	}
+}
+
+func (r *FloatTopReducer) AggregateFloat(p *FloatPoint) {
+	if r.h.Len() == cap(r.h.points) {
+		// Compare the minimum point and the aggregated point. If our value is
+		// larger, replace the current min value.
+		if !r.h.cmp(&r.h.points[0], p) {
+			return
+		}
+		r.h.points[0] = *p
+		heap.Fix(r.h, 0)
+		return
+	}
+	heap.Push(r.h, *p)
+}
+
+func (r *FloatTopReducer) Emit() []FloatPoint {
+	// Ensure the points are sorted with the maximum value last. While the
+	// first point may be the minimum value, the rest is not guaranteed to be
+	// in any particular order while it is a heap.
+	points := make([]FloatPoint, len(r.h.points))
+	for i, p := range r.h.points {
+		p.Aggregated = 0
+		points[i] = p
+	}
+	h := floatPointsByFunc{points: points, cmp: r.h.cmp}
+	sort.Sort(sort.Reverse(&h))
+	return points
+}
+
+type IntegerTopReducer struct {
+	h *integerPointsByFunc
+}
+
+func NewIntegerTopReducer(n int) *IntegerTopReducer {
+	return &IntegerTopReducer{
+		h: integerPointsSortBy(make([]IntegerPoint, 0, n), func(a, b *IntegerPoint) bool {
+			if a.Value != b.Value {
+				return a.Value < b.Value
+			}
+			return a.Time > b.Time
+		}),
+	}
+}
+
+func (r *IntegerTopReducer) AggregateInteger(p *IntegerPoint) {
+	if r.h.Len() == cap(r.h.points) {
+		// Compare the minimum point and the aggregated point. If our value is
+		// larger, replace the current min value.
+		if !r.h.cmp(&r.h.points[0], p) {
+			return
+		}
+		r.h.points[0] = *p
+		heap.Fix(r.h, 0)
+		return
+	}
+	heap.Push(r.h, *p)
+}
+
+func (r *IntegerTopReducer) Emit() []IntegerPoint {
+	// Ensure the points are sorted with the maximum value last. While the
+	// first point may be the minimum value, the rest is not guaranteed to be
+	// in any particular order while it is a heap.
+	points := make([]IntegerPoint, len(r.h.points))
+	for i, p := range r.h.points {
+		p.Aggregated = 0
+		points[i] = p
+	}
+	h := integerPointsByFunc{points: points, cmp: r.h.cmp}
+	sort.Sort(sort.Reverse(&h))
+	return points
+}
+
+type FloatBottomReducer struct {
+	h *floatPointsByFunc
+}
+
+func NewFloatBottomReducer(n int) *FloatBottomReducer {
+	return &FloatBottomReducer{
+		h: floatPointsSortBy(make([]FloatPoint, 0, n), func(a, b *FloatPoint) bool {
+			if a.Value != b.Value {
+				return a.Value > b.Value
+			}
+			return a.Time > b.Time
+		}),
+	}
+}
+
+func (r *FloatBottomReducer) AggregateFloat(p *FloatPoint) {
+	if r.h.Len() == cap(r.h.points) {
+		// Compare the minimum point and the aggregated point. If our value is
+		// larger, replace the current min value.
+		if !r.h.cmp(&r.h.points[0], p) {
+			return
+		}
+		r.h.points[0] = *p
+		heap.Fix(r.h, 0)
+		return
+	}
+	heap.Push(r.h, *p)
+}
+
+func (r *FloatBottomReducer) Emit() []FloatPoint {
+	// Ensure the points are sorted with the maximum value last. While the
+	// first point may be the minimum value, the rest is not guaranteed to be
+	// in any particular order while it is a heap.
+	points := make([]FloatPoint, len(r.h.points))
+	for i, p := range r.h.points {
+		p.Aggregated = 0
+		points[i] = p
+	}
+	h := floatPointsByFunc{points: points, cmp: r.h.cmp}
+	sort.Sort(sort.Reverse(&h))
+	return points
+}
+
+type IntegerBottomReducer struct {
+	h *integerPointsByFunc
+}
+
+func NewIntegerBottomReducer(n int) *IntegerBottomReducer {
+	return &IntegerBottomReducer{
+		h: integerPointsSortBy(make([]IntegerPoint, 0, n), func(a, b *IntegerPoint) bool {
+			if a.Value != b.Value {
+				return a.Value > b.Value
+			}
+			return a.Time > b.Time
+		}),
+	}
+}
+
+func (r *IntegerBottomReducer) AggregateInteger(p *IntegerPoint) {
+	if r.h.Len() == cap(r.h.points) {
+		// Compare the minimum point and the aggregated point. If our value is
+		// larger, replace the current min value.
+		if !r.h.cmp(&r.h.points[0], p) {
+			return
+		}
+		r.h.points[0] = *p
+		heap.Fix(r.h, 0)
+		return
+	}
+	heap.Push(r.h, *p)
+}
+
+func (r *IntegerBottomReducer) Emit() []IntegerPoint {
+	// Ensure the points are sorted with the maximum value last. While the
+	// first point may be the minimum value, the rest is not guaranteed to be
+	// in any particular order while it is a heap.
+	points := make([]IntegerPoint, len(r.h.points))
+	for i, p := range r.h.points {
+		p.Aggregated = 0
+		points[i] = p
+	}
+	h := integerPointsByFunc{points: points, cmp: r.h.cmp}
+	sort.Sort(sort.Reverse(&h))
+	return points
+}
