@@ -3,6 +3,7 @@ package models_test
 import (
 	"bytes"
 	"fmt"
+	"io"
 	"math"
 	"math/rand"
 	"reflect"
@@ -26,6 +27,8 @@ var (
 	}
 	maxFloat64 = strconv.FormatFloat(math.MaxFloat64, 'f', 1, 64)
 	minFloat64 = strconv.FormatFloat(-math.MaxFloat64, 'f', 1, 64)
+
+	sink interface{}
 )
 
 func TestMarshal(t *testing.T) {
@@ -37,6 +40,16 @@ func TestMarshal(t *testing.T) {
 	}
 }
 
+func TestTags_HashKey(t *testing.T) {
+	tags = models.NewTags(map[string]string{"A FOO": "bar", "APPLE": "orange", "host": "serverA", "region": "uswest"})
+	got := tags.HashKey()
+	if exp := ",A\\ FOO=bar,APPLE=orange,host=serverA,region=uswest"; string(got) != exp {
+		t.Log("got: ", string(got))
+		t.Log("exp: ", exp)
+		t.Error("invalid match")
+	}
+}
+
 func BenchmarkMarshal(b *testing.B) {
 	for i := 0; i < b.N; i++ {
 		tags.HashKey()
@@ -68,7 +81,7 @@ func TestPoint_AppendString(t *testing.T) {
 
 func testPoint_cube(t *testing.T, f func(p models.Point)) {
 	// heard of a table-driven test? let's make a cube-driven test...
-	tagList := []models.Tags{nil, {models.Tag{Key: []byte("foo"), Value: []byte("bar")}}, tags}
+	tagList := []models.Tags{nil, {models.NewTag([]byte("foo"), []byte("bar"))}, tags}
 	fieldList := []models.Fields{{"a": 42.0}, {"a": 42, "b": "things"}, fields}
 	timeList := []time.Time{time.Time{}, time.Unix(0, 0), time.Unix(-34526, 0), time.Unix(231845, 0), time.Now()}
 
@@ -87,6 +100,38 @@ func testPoint_cube(t *testing.T, f func(p models.Point)) {
 	}
 }
 
+func TestTag_Clone(t *testing.T) {
+	tag := models.NewTag([]byte("key"), []byte("value"))
+
+	c := tag.Clone()
+
+	if &c.Key == &tag.Key || !bytes.Equal(c.Key, tag.Key) {
+		t.Fatalf("key %s should have been a clone of %s", c.Key, tag.Key)
+	}
+
+	if &c.Value == &tag.Value || !bytes.Equal(c.Value, tag.Value) {
+		t.Fatalf("value %s should have been a clone of %s", c.Value, tag.Value)
+	}
+}
+
+func TestTags_Clone(t *testing.T) {
+	tags := models.NewTags(map[string]string{"k1": "v1", "k2": "v2", "k3": "v3"})
+
+	clone := tags.Clone()
+
+	for i := range tags {
+		tag := tags[i]
+		c := clone[i]
+		if &c.Key == &tag.Key || !bytes.Equal(c.Key, tag.Key) {
+			t.Fatalf("key %s should have been a clone of %s", c.Key, tag.Key)
+		}
+
+		if &c.Value == &tag.Value || !bytes.Equal(c.Value, tag.Value) {
+			t.Fatalf("value %s should have been a clone of %s", c.Value, tag.Value)
+		}
+	}
+}
+
 var p models.Point
 
 func BenchmarkNewPoint(b *testing.B) {
@@ -96,6 +141,25 @@ func BenchmarkNewPoint(b *testing.B) {
 	}
 }
 
+func BenchmarkNewPointFromBinary(b *testing.B) {
+	pts, err := models.ParsePointsString("cpu value1=1.0,value2=1.0,value3=3.0,value4=4,value5=\"five\" 1000000000")
+	if err != nil {
+		b.Fatalf("unexpected error ParsePointsString: %v", err)
+	}
+
+	bytes, err := pts[0].MarshalBinary()
+	if err != nil {
+		b.Fatalf("unexpected error MarshalBinary: %v", err)
+	}
+
+	for i := 0; i < b.N; i++ {
+		_, err := models.NewPointFromBytes(bytes)
+		if err != nil {
+			b.Fatalf("unexpected error NewPointsFromBytes: %v", err)
+		}
+	}
+}
+
 func BenchmarkParsePointNoTags5000(b *testing.B) {
 	var batch [5000]string
 	for i := 0; i < len(batch); i++ {
@@ -244,7 +308,11 @@ func test(t *testing.T, line string, point TestPoint) {
 	}
 
 	for name, value := range point.RawFields {
-		val := pts[0].Fields()[name]
+		fields, err := pts[0].Fields()
+		if err != nil {
+			t.Fatal(err)
+		}
+		val := fields[name]
 		expfval, ok := val.(float64)
 
 		if ok && math.IsNaN(expfval) {
@@ -252,8 +320,9 @@ func test(t *testing.T, line string, point TestPoint) {
 			if ok && !math.IsNaN(gotfval) {
 				t.Errorf(`ParsePoints("%s") field '%s' mismatch. exp NaN`, line, name)
 			}
-		} else if !reflect.DeepEqual(pts[0].Fields()[name], value) {
-			t.Errorf(`ParsePoints("%s") field '%s' mismatch. got %[3]v (%[3]T), exp %[4]v (%[4]T)`, line, name, pts[0].Fields()[name], value)
+		}
+		if !reflect.DeepEqual(val, value) {
+			t.Errorf(`ParsePoints("%s") field '%s' mismatch. got %[3]v (%[3]T), exp %[4]v (%[4]T)`, line, name, val, value)
 		}
 	}
 
@@ -475,7 +544,11 @@ func TestParsePointMaxInt64(t *testing.T) {
 	if err != nil {
 		t.Fatalf(`ParsePoints("%s") mismatch. got %v, exp nil`, `cpu,host=serverA,region=us-west value=9223372036854775807i`, err)
 	}
-	if exp, got := int64(9223372036854775807), p[0].Fields()["value"].(int64); exp != got {
+	fields, err := p[0].Fields()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if exp, got := int64(9223372036854775807), fields["value"].(int64); exp != got {
 		t.Fatalf("ParsePoints Value mismatch. \nexp: %v\ngot: %v", exp, got)
 	}
 
@@ -613,7 +686,11 @@ func TestParsePointFloatScientific(t *testing.T) {
 		t.Errorf(`ParsePoints("%s") mismatch. got %v, exp nil`, `cpu,host=serverA,region=us-west value=1.0e4`, err)
 	}
 
-	if pts[0].Fields()["value"] != 1e4 {
+	fields, err := pts[0].Fields()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if fields["value"] != 1e4 {
 		t.Errorf(`ParsePoints("%s") mismatch. got %v, exp nil`, `cpu,host=serverA,region=us-west value=1e4`, err)
 	}
 }
@@ -629,7 +706,11 @@ func TestParsePointFloatScientificUpper(t *testing.T) {
 		t.Errorf(`ParsePoints("%s") mismatch. got %v, exp nil`, `cpu,host=serverA,region=us-west value=1.0E4`, err)
 	}
 
-	if pts[0].Fields()["value"] != 1e4 {
+	fields, err := pts[0].Fields()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if fields["value"] != 1e4 {
 		t.Errorf(`ParsePoints("%s") mismatch. got %v, exp nil`, `cpu,host=serverA,region=us-west value=1E4`, err)
 	}
 }
@@ -690,15 +771,23 @@ func TestParsePointWhitespace(t *testing.T) {
 			t.Fatalf("[Example %d] got %d points, expected %d", i, got, exp)
 		}
 
-		if got, exp := pts[0].Name(), expPoint.Name(); got != exp {
+		if got, exp := string(pts[0].Name()), string(expPoint.Name()); got != exp {
 			t.Fatalf("[Example %d] got %v measurement, expected %v", i, got, exp)
 		}
 
-		if got, exp := len(pts[0].Fields()), len(expPoint.Fields()); got != exp {
+		fields, err := pts[0].Fields()
+		if err != nil {
+			t.Fatal(err)
+		}
+		eFields, err := expPoint.Fields()
+		if err != nil {
+			t.Fatal(err)
+		}
+		if got, exp := len(fields), len(eFields); got != exp {
 			t.Fatalf("[Example %d] got %d fields, expected %d", i, got, exp)
 		}
 
-		if got, exp := pts[0].Fields()["value"], expPoint.Fields()["value"]; got != exp {
+		if got, exp := fields["value"], eFields["value"]; got != exp {
 			t.Fatalf(`[Example %d] got %v for field "value", expected %v`, i, got, exp)
 		}
 
@@ -1447,16 +1536,20 @@ func TestParsePointIntsFloats(t *testing.T) {
 	}
 	pt := pts[0]
 
-	if _, ok := pt.Fields()["int"].(int64); !ok {
-		t.Errorf("ParsePoint() int field mismatch: got %T, exp %T", pt.Fields()["int"], int64(10))
+	fields, err := pt.Fields()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if _, ok := fields["int"].(int64); !ok {
+		t.Errorf("ParsePoint() int field mismatch: got %T, exp %T", fields["int"], int64(10))
 	}
 
-	if _, ok := pt.Fields()["float"].(float64); !ok {
-		t.Errorf("ParsePoint() float field mismatch: got %T, exp %T", pt.Fields()["float64"], float64(11.0))
+	if _, ok := fields["float"].(float64); !ok {
+		t.Errorf("ParsePoint() float field mismatch: got %T, exp %T", fields["float64"], float64(11.0))
 	}
 
-	if _, ok := pt.Fields()["float2"].(float64); !ok {
-		t.Errorf("ParsePoint() float field mismatch: got %T, exp %T", pt.Fields()["float64"], float64(12.1))
+	if _, ok := fields["float2"].(float64); !ok {
+		t.Errorf("ParsePoint() float field mismatch: got %T, exp %T", fields["float64"], float64(12.1))
 	}
 }
 
@@ -1728,7 +1821,11 @@ func TestNewPointUnhandledType(t *testing.T) {
 		t.Errorf("NewPoint().String() mismatch.\ngot %v\nexp %v", pt.String(), exp)
 	}
 
-	if exp := "1970-01-01 00:00:00 +0000 UTC"; pt.Fields()["value"] != exp {
+	fields, err := pt.Fields()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if exp := "1970-01-01 00:00:00 +0000 UTC"; fields["value"] != exp {
 		t.Errorf("NewPoint().String() mismatch.\ngot %v\nexp %v", pt.String(), exp)
 	}
 }
@@ -1894,8 +1991,11 @@ cpu value=2 1`
 		t.Fatalf("failed to write points: %s", err.Error())
 	}
 
-	pointFields := points[0].Fields()
-	value, ok := pointFields["\"a"]
+	fields, err := points[0].Fields()
+	if err != nil {
+		t.Fatal(err)
+	}
+	value, ok := fields["\"a"]
 	if !ok {
 		t.Fatalf("expected to parse field '\"a'")
 	}
@@ -1947,6 +2047,13 @@ func TestNewPointsWithBytesWithCorruptData(t *testing.T) {
 	}
 }
 
+func TestNewPointsWithShortBuffer(t *testing.T) {
+	_, err := models.NewPointFromBytes([]byte{0, 0, 0, 3, 4})
+	if err != io.ErrShortBuffer {
+		t.Fatalf("NewPointFromBytes: got: (%v, %v), expected: (nil, error)", p, err)
+	}
+}
+
 func TestNewPointsRejectsEmptyFieldNames(t *testing.T) {
 	if _, err := models.NewPoint("foo", nil, models.Fields{"": 1}, time.Now()); err == nil {
 		t.Fatalf("new point with empty field name. got: nil, expected: error")
@@ -1955,23 +2062,30 @@ func TestNewPointsRejectsEmptyFieldNames(t *testing.T) {
 
 func TestNewPointsRejectsMaxKey(t *testing.T) {
 	var key string
-	for i := 0; i < 65536; i++ {
+	// tsm field key is point key, separator (4 bytes) and field
+	for i := 0; i < models.MaxKeyLength-len("value")-4; i++ {
 		key += "a"
 	}
 
-	if _, err := models.NewPoint(key, nil, models.Fields{"value": 1}, time.Now()); err == nil {
+	// Test max key len
+	if _, err := models.NewPoint(key, nil, models.Fields{"value": 1, "ok": 2.0}, time.Now()); err != nil {
+		t.Fatalf("new point with max key. got: %v, expected: nil", err)
+	}
+
+	if _, err := models.ParsePointsString(fmt.Sprintf("%v value=1,ok=2.0", key)); err != nil {
+		t.Fatalf("parse point with max key. got: %v, expected: nil", err)
+	}
+
+	// Test 1 byte over max key len
+	key += "a"
+	if _, err := models.NewPoint(key, nil, models.Fields{"value": 1, "ok": 2.0}, time.Now()); err == nil {
 		t.Fatalf("new point with max key. got: nil, expected: error")
 	}
 
-	if _, err := models.ParsePointsString(fmt.Sprintf("%v value=1", key)); err == nil {
+	if _, err := models.ParsePointsString(fmt.Sprintf("%v value=1,ok=2.0", key)); err == nil {
 		t.Fatalf("parse point with max key. got: nil, expected: error")
 	}
-}
 
-func TestParseKeyEmpty(t *testing.T) {
-	if _, _, err := models.ParseKey(nil); err != nil {
-		t.Fatalf("unexpected error: %v", err)
-	}
 }
 
 func TestPoint_FieldIterator_Simple(t *testing.T) {
@@ -1995,8 +2109,12 @@ func TestPoint_FieldIterator_Simple(t *testing.T) {
 		t.Fatalf("'42i' should be an Integer, got %v", fi.Type())
 	}
 
-	if fi.IntegerValue() != 42 {
-		t.Fatalf("'42i' should be 42, got %d", fi.IntegerValue())
+	iv, err := fi.IntegerValue()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if exp, got := int64(42), iv; exp != got {
+		t.Fatalf("'42i' should be %d, got %d", exp, got)
 	}
 
 	if !fi.Next() {
@@ -2007,8 +2125,12 @@ func TestPoint_FieldIterator_Simple(t *testing.T) {
 		t.Fatalf("'42' should be a Float, got %v", fi.Type())
 	}
 
-	if fi.FloatValue() != 42.0 {
-		t.Fatalf("'42' should be %f, got %f", 42.0, fi.FloatValue())
+	fv, err := fi.FloatValue()
+	if err != nil {
+		t.Fatal(err)
+	}
+	if exp, got := 42.0, fv; exp != got {
+		t.Fatalf("'42' should be %f, got %f", exp, got)
 	}
 
 	if fi.Next() {
@@ -2020,20 +2142,24 @@ func toFields(fi models.FieldIterator) models.Fields {
 	m := make(models.Fields)
 	for fi.Next() {
 		var v interface{}
+		var err error
 		switch fi.Type() {
 		case models.Float:
-			v = fi.FloatValue()
+			v, err = fi.FloatValue()
 		case models.Integer:
-			v = fi.IntegerValue()
+			v, err = fi.IntegerValue()
 		case models.String:
 			v = fi.StringValue()
 		case models.Boolean:
-			v = fi.BooleanValue()
+			v, err = fi.BooleanValue()
 		case models.Empty:
 			v = nil
 		default:
 			panic("unknown type")
 		}
+		if err != nil {
+			panic(err)
+		}
 		m[string(fi.FieldKey())] = v
 	}
 	return m
@@ -2056,7 +2182,10 @@ m a=2i,b=3i,c=true,d="stuff",e=-0.23,f=123.456
 	}
 
 	for _, p := range points {
-		exp := p.Fields()
+		exp, err := p.Fields()
+		if err != nil {
+			t.Fatal(err)
+		}
 		got := toFields(p.FieldIterator())
 
 		if !reflect.DeepEqual(got, exp) {
@@ -2065,106 +2194,60 @@ m a=2i,b=3i,c=true,d="stuff",e=-0.23,f=123.456
 	}
 }
 
-func TestPoint_FieldIterator_Delete_Begin(t *testing.T) {
-	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
-	if err != nil || len(points) != 1 {
-		t.Fatal("failed parsing point")
+func TestEscapeStringField(t *testing.T) {
+	cases := []struct {
+		in     string
+		expOut string
+	}{
+		{in: "abcdefg", expOut: "abcdefg"},
+		{in: `one double quote " .`, expOut: `one double quote \" .`},
+		{in: `quote " then backslash \ .`, expOut: `quote \" then backslash \\ .`},
+		{in: `backslash \ then quote " .`, expOut: `backslash \\ then quote \" .`},
 	}
 
-	fi := points[0].FieldIterator()
-	fi.Next() // a
-	fi.Delete()
+	for _, c := range cases {
+		// Unescapes as expected.
+		got := models.EscapeStringField(c.in)
+		if got != c.expOut {
+			t.Errorf("unexpected result from EscapeStringField(%s)\ngot [%s]\nexp [%s]\n", c.in, got, c.expOut)
+			continue
+		}
 
-	fi.Reset()
-
-	got := toFields(fi)
-	exp := models.Fields{"b": float64(2), "c": float64(3)}
-
-	if !reflect.DeepEqual(got, exp) {
-		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+		pointLine := fmt.Sprintf(`t s="%s"`, got)
+		test(t, pointLine, NewTestPoint(
+			"t",
+			models.NewTags(nil),
+			models.Fields{"s": c.in},
+			time.Unix(0, 0),
+		))
 	}
 }
 
-func TestPoint_FieldIterator_Delete_Middle(t *testing.T) {
-	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
-	if err != nil || len(points) != 1 {
-		t.Fatal("failed parsing point")
-	}
-
-	fi := points[0].FieldIterator()
-	fi.Next() // a
-	fi.Next() // b
-	fi.Delete()
-
-	fi.Reset()
-
-	got := toFields(fi)
-	exp := models.Fields{"a": float64(1), "c": float64(3)}
-
-	if !reflect.DeepEqual(got, exp) {
-		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+func BenchmarkEscapeStringField_Plain(b *testing.B) {
+	s := "nothing special"
+	for i := 0; i < b.N; i++ {
+		sink = models.EscapeStringField(s)
 	}
 }
 
-func TestPoint_FieldIterator_Delete_End(t *testing.T) {
-	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
-	if err != nil || len(points) != 1 {
-		t.Fatal("failed parsing point")
-	}
-
-	fi := points[0].FieldIterator()
-	fi.Next() // a
-	fi.Next() // b
-	fi.Next() // c
-	fi.Delete()
-
-	fi.Reset()
-
-	got := toFields(fi)
-	exp := models.Fields{"a": float64(1), "b": float64(2)}
-
-	if !reflect.DeepEqual(got, exp) {
-		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+func BenchmarkEscapeString_Quotes(b *testing.B) {
+	s := `Hello, "world"`
+	for i := 0; i < b.N; i++ {
+		sink = models.EscapeStringField(s)
 	}
 }
 
-func TestPoint_FieldIterator_Delete_Nothing(t *testing.T) {
-	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
-	if err != nil || len(points) != 1 {
-		t.Fatal("failed parsing point")
-	}
-
-	fi := points[0].FieldIterator()
-	fi.Delete()
-
-	fi.Reset()
-
-	got := toFields(fi)
-	exp := models.Fields{"a": float64(1), "b": float64(2), "c": float64(3)}
-
-	if !reflect.DeepEqual(got, exp) {
-		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+func BenchmarkEscapeString_Backslashes(b *testing.B) {
+	s := `C:\windows\system32`
+	for i := 0; i < b.N; i++ {
+		sink = models.EscapeStringField(s)
 	}
 }
 
-func TestPoint_FieldIterator_Delete_Twice(t *testing.T) {
-	points, err := models.ParsePointsString(`m a=1,b=2,c=3`)
-	if err != nil || len(points) != 1 {
-		t.Fatal("failed parsing point")
-	}
-
-	fi := points[0].FieldIterator()
-	fi.Next() // a
-	fi.Next() // b
-	fi.Delete()
-	fi.Delete() // no-op
-
-	fi.Reset()
-
-	got := toFields(fi)
-	exp := models.Fields{"a": float64(1), "c": float64(3)}
-
-	if !reflect.DeepEqual(got, exp) {
-		t.Fatalf("Delete failed, got %#v, exp %#v", got, exp)
+func BenchmarkEscapeString_QuotesAndBackslashes(b *testing.B) {
+	s1 := `a quote " then backslash \ .`
+	s2 := `a backslash \ then quote " .`
+	for i := 0; i < b.N; i++ {
+		sink = [...]string{models.EscapeStringField(s1), models.EscapeStringField(s2)}
 	}
 }
