@@ -1,12 +1,11 @@
+// Package monitor provides a service and associated functionality
+// for InfluxDB to self-monitor internal statistics and diagnostics.
 package monitor // import "github.com/influxdata/influxdb/monitor"
 
 import (
-	"bytes"
 	"errors"
 	"expvar"
 	"fmt"
-	"io"
-	"log"
 	"os"
 	"runtime"
 	"sort"
@@ -17,12 +16,18 @@ import (
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/monitor/diagnostics"
 	"github.com/influxdata/influxdb/services/meta"
+	"github.com/uber-go/zap"
 )
 
 // Policy constants.
 const (
-	MonitorRetentionPolicy         = "monitor"
+	// Name of the retention policy used by the monitor service.
+	MonitorRetentionPolicy = "monitor"
+
+	// Duration of the monitor retention policy.
 	MonitorRetentionPolicyDuration = 7 * 24 * time.Hour
+
+	// Default replication factor to set on the monitor retention policy.
 	MonitorRetentionPolicyReplicaN = 1
 )
 
@@ -43,13 +48,10 @@ type Monitor struct {
 	done              chan struct{}
 	storeCreated      bool
 	storeEnabled      bool
-	storeAddress      string
 
-	storeDatabase          string
-	storeRetentionPolicy   string
-	storeRetentionDuration time.Duration
-	storeReplicationFactor int
-	storeInterval          time.Duration
+	storeDatabase        string
+	storeRetentionPolicy string
+	storeInterval        time.Duration
 
 	MetaClient interface {
 		CreateDatabaseWithRetentionPolicy(name string, spec *meta.RetentionPolicySpec) (*meta.DatabaseInfo, error)
@@ -59,10 +61,10 @@ type Monitor struct {
 	// Writer for pushing stats back into the database.
 	PointsWriter PointsWriter
 
-	Logger *log.Logger
+	Logger zap.Logger
 }
 
-// PointsWriter is a simplified interface for writing the points the monitor gathers
+// PointsWriter is a simplified interface for writing the points the monitor gathers.
 type PointsWriter interface {
 	WritePoints(database, retentionPolicy string, points models.Points) error
 }
@@ -77,10 +79,11 @@ func New(r Reporter, c Config) *Monitor {
 		storeDatabase:        c.StoreDatabase,
 		storeInterval:        time.Duration(c.StoreInterval),
 		storeRetentionPolicy: MonitorRetentionPolicy,
-		Logger:               log.New(os.Stderr, "[monitor] ", log.LstdFlags),
+		Logger:               zap.New(zap.NullEncoder()),
 	}
 }
 
+// open returns whether the monitor service is open.
 func (m *Monitor) open() bool {
 	m.mu.Lock()
 	defer m.mu.Unlock()
@@ -91,11 +94,11 @@ func (m *Monitor) open() bool {
 // for identification purpose.
 func (m *Monitor) Open() error {
 	if m.open() {
-		m.Logger.Println("Monitor is already open")
+		m.Logger.Info("Monitor is already open")
 		return nil
 	}
 
-	m.Logger.Printf("Starting monitor system")
+	m.Logger.Info("Starting monitor system")
 
 	// Self-register various stats and diagnostics.
 	m.RegisterDiagnosticsClient("build", &build{
@@ -122,14 +125,24 @@ func (m *Monitor) Open() error {
 	return nil
 }
 
+func (m *Monitor) writePoints(p models.Points) error {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	if err := m.PointsWriter.WritePoints(m.storeDatabase, m.storeRetentionPolicy, p); err != nil {
+		m.Logger.Info(fmt.Sprintf("failed to store statistics: %s", err))
+	}
+	return nil
+}
+
 // Close closes the monitor system.
 func (m *Monitor) Close() error {
 	if !m.open() {
-		m.Logger.Println("Monitor is already closed.")
+		m.Logger.Info("Monitor is already closed.")
 		return nil
 	}
 
-	m.Logger.Println("shutting down monitor system")
+	m.Logger.Info("shutting down monitor system")
 	m.mu.Lock()
 	close(m.done)
 	m.mu.Unlock()
@@ -155,7 +168,7 @@ func (m *Monitor) SetGlobalTag(key string, value interface{}) {
 	m.mu.Unlock()
 }
 
-// RemoteWriterConfig represents the configuration of a remote writer
+// RemoteWriterConfig represents the configuration of a remote writer.
 type RemoteWriterConfig struct {
 	RemoteAddr string
 	NodeID     string
@@ -178,10 +191,9 @@ func (m *Monitor) SetPointsWriter(pw PointsWriter) error {
 	return m.Open()
 }
 
-// SetLogOutput sets the writer to which all logs are written. It must not be
-// called after Open is called.
-func (m *Monitor) SetLogOutput(w io.Writer) {
-	m.Logger = log.New(w, "[monitor] ", log.LstdFlags)
+// WithLogger sets the logger for the Monitor.
+func (m *Monitor) WithLogger(log zap.Logger) {
+	m.Logger = log.With(zap.String("service", "monitor"))
 }
 
 // RegisterDiagnosticsClient registers a diagnostics client with the given name and tags.
@@ -189,7 +201,7 @@ func (m *Monitor) RegisterDiagnosticsClient(name string, client diagnostics.Clie
 	m.mu.Lock()
 	defer m.mu.Unlock()
 	m.diagRegistrations[name] = client
-	m.Logger.Printf(`'%s' registered for diagnostics monitoring`, name)
+	m.Logger.Info(fmt.Sprintf(`'%s' registered for diagnostics monitoring`, name))
 }
 
 // DeregisterDiagnosticsClient deregisters a diagnostics client by name.
@@ -211,10 +223,7 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*Statistic, error) {
 		}
 
 		statistic := &Statistic{
-			Statistic: models.Statistic{
-				Tags:   make(map[string]string),
-				Values: make(map[string]interface{}),
-			},
+			Statistic: models.NewStatistic(""),
 		}
 
 		// Add any supplied tags.
@@ -279,11 +288,7 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*Statistic, error) {
 
 	// Add Go memstats.
 	statistic := &Statistic{
-		Statistic: models.Statistic{
-			Name:   "runtime",
-			Tags:   make(map[string]string),
-			Values: make(map[string]interface{}),
-		},
+		Statistic: models.NewStatistic("runtime"),
 	}
 
 	// Add any supplied tags to Go memstats
@@ -360,8 +365,8 @@ func (m *Monitor) createInternalStorage() {
 		}
 
 		if _, err := m.MetaClient.CreateDatabaseWithRetentionPolicy(m.storeDatabase, &spec); err != nil {
-			m.Logger.Printf("failed to create database '%s', failed to create storage: %s",
-				m.storeDatabase, err.Error())
+			m.Logger.Info(fmt.Sprintf("failed to create database '%s', failed to create storage: %s",
+				m.storeDatabase, err.Error()))
 			return
 		}
 	}
@@ -388,8 +393,8 @@ func (m *Monitor) waitUntilInterval(d time.Duration) error {
 // storeStatistics writes the statistics to an InfluxDB system.
 func (m *Monitor) storeStatistics() {
 	defer m.wg.Done()
-	m.Logger.Printf("Storing statistics in database '%s' retention policy '%s', at interval %s",
-		m.storeDatabase, m.storeRetentionPolicy, m.storeInterval)
+	m.Logger.Info(fmt.Sprintf("Storing statistics in database '%s' retention policy '%s', at interval %s",
+		m.storeDatabase, m.storeRetentionPolicy, m.storeInterval))
 
 	hostname, _ := os.Hostname()
 	m.SetGlobalTag("hostname", hostname)
@@ -415,30 +420,32 @@ func (m *Monitor) storeStatistics() {
 
 			stats, err := m.Statistics(m.globalTags)
 			if err != nil {
-				m.Logger.Printf("failed to retrieve registered statistics: %s", err)
+				m.Logger.Info(fmt.Sprintf("failed to retrieve registered statistics: %s", err))
 				return
 			}
 
-			points := make(models.Points, 0, len(stats))
+			// Write all stats in batches
+			batch := make(models.Points, 0, 5000)
 			for _, s := range stats {
-				pt, err := models.NewPoint(s.Name, s.Tags, s.Values, now)
+				pt, err := models.NewPoint(s.Name, models.NewTags(s.Tags), s.Values, now)
 				if err != nil {
-					m.Logger.Printf("Dropping point %v: %v", s.Name, err)
+					m.Logger.Info(fmt.Sprintf("Dropping point %v: %v", s.Name, err))
 					return
 				}
-				points = append(points, pt)
+				batch = append(batch, pt)
+				if len(batch) == cap(batch) {
+					m.writePoints(batch)
+					batch = batch[:0]
+
+				}
 			}
 
-			func() {
-				m.mu.RLock()
-				defer m.mu.RUnlock()
-
-				if err := m.PointsWriter.WritePoints(m.storeDatabase, m.storeRetentionPolicy, points); err != nil {
-					m.Logger.Printf("failed to store statistics: %s", err)
-				}
-			}()
+			// Write the last batch
+			if len(batch) > 0 {
+				m.writePoints(batch)
+			}
 		case <-m.done:
-			m.Logger.Printf("terminating storage of statistics")
+			m.Logger.Info(fmt.Sprintf("terminating storage of statistics"))
 			return
 		}
 	}
@@ -449,7 +456,7 @@ type Statistic struct {
 	models.Statistic
 }
 
-// valueNames returns a sorted list of the value names, if any.
+// ValueNames returns a sorted list of the value names, if any.
 func (s *Statistic) ValueNames() []string {
 	a := make([]string, 0, len(s.Values))
 	for k := range s.Values {
@@ -459,32 +466,16 @@ func (s *Statistic) ValueNames() []string {
 	return a
 }
 
+// Statistics is a slice of sortable statistics.
 type Statistics []*Statistic
 
+// Len implements sort.Interface.
 func (a Statistics) Len() int { return len(a) }
+
+// Less implements sort.Interface.
 func (a Statistics) Less(i, j int) bool {
-	if a[i].Name != a[j].Name {
-		return a[i].Name < a[j].Name
-	}
-	return bytes.Compare(a[i].Tags.HashKey(), a[j].Tags.HashKey()) < 0
+	return a[i].Name < a[j].Name
 }
+
+// Swap implements sort.Interface.
 func (a Statistics) Swap(i, j int) { a[i], a[j] = a[j], a[i] }
-
-// DiagnosticsFromMap returns a Diagnostics from a map.
-func DiagnosticsFromMap(m map[string]interface{}) *diagnostics.Diagnostics {
-	// Display columns in deterministic order.
-	sortedKeys := make([]string, 0, len(m))
-	for k := range m {
-		sortedKeys = append(sortedKeys, k)
-	}
-	sort.Strings(sortedKeys)
-
-	d := diagnostics.NewDiagnostics(sortedKeys)
-	row := make([]interface{}, len(sortedKeys))
-	for i, k := range sortedKeys {
-		row[i] = m[k]
-	}
-	d.AddRow(row)
-
-	return d
-}
