@@ -5,8 +5,8 @@ import (
 	"compress/gzip"
 	"encoding/json"
 	"errors"
+	"fmt"
 	"io"
-	"log"
 	"net"
 	"net/http"
 	"sync"
@@ -15,22 +15,24 @@ import (
 
 	"github.com/influxdata/influxdb"
 	"github.com/influxdata/influxdb/models"
+	"github.com/uber-go/zap"
 )
 
-// Handler is an http.Handler for the service.
+// Handler is an http.Handler for the OpenTSDB service.
 type Handler struct {
 	Database        string
 	RetentionPolicy string
 
 	PointsWriter interface {
-		WritePoints(database, retentionPolicy string, consistencyLevel models.ConsistencyLevel, points []models.Point) error
+		WritePointsPrivileged(database, retentionPolicy string, consistencyLevel models.ConsistencyLevel, points []models.Point) error
 	}
 
-	Logger *log.Logger
+	Logger zap.Logger
 
 	stats *Statistics
 }
 
+// ServeHTTP handles an HTTP request of the OpenTSDB REST API.
 func (h *Handler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	switch r.URL.Path {
 	case "/api/metadata/put":
@@ -42,7 +44,7 @@ func (h *Handler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	}
 }
 
-// ServeHTTP implements OpenTSDB's HTTP /api/put endpoint
+// servePut implements OpenTSDB's HTTP /api/put endpoint.
 func (h *Handler) servePut(w http.ResponseWriter, r *http.Request) {
 	defer r.Body.Close()
 
@@ -114,7 +116,7 @@ func (h *Handler) servePut(w http.ResponseWriter, r *http.Request) {
 
 		pt, err := models.NewPoint(p.Metric, models.NewTags(p.Tags), map[string]interface{}{"value": p.Value}, ts)
 		if err != nil {
-			h.Logger.Printf("Dropping point %v: %v", p.Metric, err)
+			h.Logger.Info(fmt.Sprintf("Dropping point %v: %v", p.Metric, err))
 			if h.stats != nil {
 				atomic.AddInt64(&h.stats.InvalidDroppedPoints, 1)
 			}
@@ -124,12 +126,12 @@ func (h *Handler) servePut(w http.ResponseWriter, r *http.Request) {
 	}
 
 	// Write points.
-	if err := h.PointsWriter.WritePoints(h.Database, h.RetentionPolicy, models.ConsistencyLevelAny, points); influxdb.IsClientError(err) {
-		h.Logger.Println("write series error: ", err)
+	if err := h.PointsWriter.WritePointsPrivileged(h.Database, h.RetentionPolicy, models.ConsistencyLevelAny, points); influxdb.IsClientError(err) {
+		h.Logger.Info(fmt.Sprint("write series error: ", err))
 		http.Error(w, "write series error: "+err.Error(), http.StatusBadRequest)
 		return
 	} else if err != nil {
-		h.Logger.Println("write series error: ", err)
+		h.Logger.Info(fmt.Sprint("write series error: ", err))
 		http.Error(w, "write series error: "+err.Error(), http.StatusInternalServerError)
 		return
 	}
