@@ -5,11 +5,15 @@ import (
 	"fmt"
 	"io"
 	"os"
+	"regexp"
 	"sort"
 	"time"
 
 	"github.com/influxdata/influxdb/influxql"
 	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/pkg/estimator"
+	"github.com/influxdata/influxdb/pkg/limiter"
+	"github.com/uber-go/zap"
 )
 
 var (
@@ -26,29 +30,53 @@ var (
 type Engine interface {
 	Open() error
 	Close() error
+	SetEnabled(enabled bool)
+	SetCompactionsEnabled(enabled bool)
 
-	SetLogOutput(io.Writer)
-	LoadMetadataIndex(shardID uint64, index *DatabaseIndex) error
+	WithLogger(zap.Logger)
 
+	LoadMetadataIndex(shardID uint64, index Index) error
+
+	CreateSnapshot() (string, error)
 	Backup(w io.Writer, basePath string, since time.Time) error
 	Restore(r io.Reader, basePath string) error
+	Import(r io.Reader, basePath string) error
 
-	CreateIterator(opt influxql.IteratorOptions) (influxql.Iterator, error)
+	CreateIterator(measurement string, opt influxql.IteratorOptions) (influxql.Iterator, error)
 	WritePoints(points []models.Point) error
-	ContainsSeries(keys []string) (map[string]bool, error)
-	DeleteSeries(keys []string) error
-	DeleteSeriesRange(keys []string, min, max int64) error
-	DeleteMeasurement(name string, seriesKeys []string) error
-	SeriesCount() (n int, err error)
-	MeasurementFields(measurement string) *MeasurementFields
-	CreateSnapshot() (string, error)
-	SetEnabled(enabled bool)
 
-	// Format will return the format for the engine
-	Format() EngineFormat
+	CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error
+	CreateSeriesListIfNotExists(keys, names [][]byte, tags []models.Tags) error
+	DeleteSeriesRange(keys [][]byte, min, max int64) error
+
+	SeriesSketches() (estimator.Sketch, estimator.Sketch, error)
+	MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error)
+	SeriesN() int64
+
+	MeasurementExists(name []byte) (bool, error)
+	MeasurementNamesByExpr(expr influxql.Expr) ([][]byte, error)
+	MeasurementNamesByRegex(re *regexp.Regexp) ([][]byte, error)
+	MeasurementFields(measurement []byte) *MeasurementFields
+	ForEachMeasurementName(fn func(name []byte) error) error
+	DeleteMeasurement(name []byte) error
+
+	// TagKeys(name []byte) ([][]byte, error)
+	HasTagKey(name, key []byte) (bool, error)
+	MeasurementTagKeysByExpr(name []byte, expr influxql.Expr) (map[string]struct{}, error)
+	MeasurementTagKeyValuesByExpr(name []byte, key []string, expr influxql.Expr, keysSorted bool) ([][]string, error)
+	ForEachMeasurementTagKey(name []byte, fn func(key []byte) error) error
+	TagKeyCardinality(name, key []byte) int
+
+	// InfluxQL iterators
+	MeasurementSeriesKeysByExpr(name []byte, condition influxql.Expr) ([][]byte, error)
+	ForEachMeasurementSeriesByExpr(name []byte, expr influxql.Expr, fn func(tags models.Tags) error) error
+	SeriesPointIterator(opt influxql.IteratorOptions) (influxql.Iterator, error)
 
 	// Statistics will return statistics relevant to this engine.
 	Statistics(tags map[string]string) []models.Statistic
+	LastModified() time.Time
+	DiskSize() int64
+	IsIdle() bool
 
 	io.WriterTo
 }
@@ -62,7 +90,7 @@ const (
 )
 
 // NewEngineFunc creates a new engine.
-type NewEngineFunc func(id uint64, path string, walPath string, options EngineOptions) Engine
+type NewEngineFunc func(id uint64, i Index, database, path string, walPath string, options EngineOptions) Engine
 
 // newEngineFuncs is a lookup of engine constructors by name.
 var newEngineFuncs = make(map[string]NewEngineFunc)
@@ -87,10 +115,10 @@ func RegisteredEngines() []string {
 
 // NewEngine returns an instance of an engine based on its format.
 // If the path does not exist then the DefaultFormat is used.
-func NewEngine(id uint64, path string, walPath string, options EngineOptions) (Engine, error) {
+func NewEngine(id uint64, i Index, database, path string, walPath string, options EngineOptions) (Engine, error) {
 	// Create a new engine
 	if _, err := os.Stat(path); os.IsNotExist(err) {
-		return newEngineFuncs[options.EngineVersion](id, path, walPath, options), nil
+		return newEngineFuncs[options.EngineVersion](id, i, database, path, walPath, options), nil
 	}
 
 	// If it's a dir then it's a tsm1 engine
@@ -109,13 +137,16 @@ func NewEngine(id uint64, path string, walPath string, options EngineOptions) (E
 		return nil, fmt.Errorf("invalid engine format: %q", format)
 	}
 
-	return fn(id, path, walPath, options), nil
+	return fn(id, i, database, path, walPath, options), nil
 }
 
 // EngineOptions represents the options used to initialize the engine.
 type EngineOptions struct {
-	EngineVersion string
-	ShardID       uint64
+	EngineVersion     string
+	IndexVersion      string
+	ShardID           uint64
+	InmemIndex        interface{} // shared in-memory index
+	CompactionLimiter limiter.Fixed
 
 	Config Config
 }
@@ -124,6 +155,10 @@ type EngineOptions struct {
 func NewEngineOptions() EngineOptions {
 	return EngineOptions{
 		EngineVersion: DefaultEngine,
+		IndexVersion:  DefaultIndex,
 		Config:        NewConfig(),
 	}
 }
+
+// NewInmemIndex returns a new "inmem" index type.
+var NewInmemIndex func(name string) (interface{}, error)
