@@ -8,7 +8,7 @@ import (
 	"testing"
 	"time"
 
-	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
 )
 
@@ -191,6 +191,155 @@ func TestCompactor_CompactFull(t *testing.T) {
 	}
 }
 
+// Ensures that a compaction will properly merge multiple TSM files
+func TestCompactor_Compact_OverlappingBlocks(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+
+	// write 3 TSM files with different data and one new point
+	a1 := tsm1.NewValue(4, 1.1)
+	a2 := tsm1.NewValue(5, 1.1)
+	a3 := tsm1.NewValue(7, 1.1)
+
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3},
+	}
+	f1 := MustWriteTSM(dir, 1, writes)
+
+	c1 := tsm1.NewValue(3, 1.2)
+	c2 := tsm1.NewValue(8, 1.2)
+	c3 := tsm1.NewValue(9, 1.2)
+
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{c1, c2, c3},
+	}
+	f3 := MustWriteTSM(dir, 3, writes)
+
+	compactor := &tsm1.Compactor{
+		Dir:       dir,
+		FileStore: &fakeFileStore{},
+		Size:      2,
+	}
+
+	compactor.Open()
+
+	files, err := compactor.CompactFast([]string{f1, f3})
+	if err != nil {
+		t.Fatalf("unexpected error writing snapshot: %v", err)
+	}
+
+	if got, exp := len(files), 1; got != exp {
+		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
+	}
+
+	r := MustOpenTSMReader(files[0])
+
+	if got, exp := r.KeyCount(), 1; got != exp {
+		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
+	}
+
+	var data = []struct {
+		key    string
+		points []tsm1.Value
+	}{
+		{"cpu,host=A#!~#value", []tsm1.Value{c1, a1, a2, a3, c2, c3}},
+	}
+
+	for _, p := range data {
+		values, err := r.ReadAll(p.key)
+		if err != nil {
+			t.Fatalf("unexpected error reading: %v", err)
+		}
+
+		if got, exp := len(values), len(p.points); got != exp {
+			t.Fatalf("values length mismatch %s: got %v, exp %v", p.key, got, exp)
+		}
+
+		for i, point := range p.points {
+			assertValueEqual(t, values[i], point)
+		}
+	}
+}
+
+// Ensures that a compaction will properly merge multiple TSM files
+func TestCompactor_Compact_OverlappingBlocksMultiple(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+
+	// write 3 TSM files with different data and one new point
+	a1 := tsm1.NewValue(4, 1.1)
+	a2 := tsm1.NewValue(5, 1.1)
+	a3 := tsm1.NewValue(7, 1.1)
+
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3},
+	}
+	f1 := MustWriteTSM(dir, 1, writes)
+
+	b1 := tsm1.NewValue(1, 1.2)
+	b2 := tsm1.NewValue(2, 1.2)
+	b3 := tsm1.NewValue(6, 1.2)
+
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{b1, b2, b3},
+	}
+	f2 := MustWriteTSM(dir, 2, writes)
+
+	c1 := tsm1.NewValue(3, 1.2)
+	c2 := tsm1.NewValue(8, 1.2)
+	c3 := tsm1.NewValue(9, 1.2)
+
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{c1, c2, c3},
+	}
+	f3 := MustWriteTSM(dir, 3, writes)
+
+	compactor := &tsm1.Compactor{
+		Dir:       dir,
+		FileStore: &fakeFileStore{},
+		Size:      2,
+	}
+
+	compactor.Open()
+
+	files, err := compactor.CompactFast([]string{f1, f2, f3})
+	if err != nil {
+		t.Fatalf("unexpected error writing snapshot: %v", err)
+	}
+
+	if got, exp := len(files), 1; got != exp {
+		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
+	}
+
+	r := MustOpenTSMReader(files[0])
+
+	if got, exp := r.KeyCount(), 1; got != exp {
+		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
+	}
+
+	var data = []struct {
+		key    string
+		points []tsm1.Value
+	}{
+		{"cpu,host=A#!~#value", []tsm1.Value{b1, b2, c1, a1, a2, b3, a3, c2, c3}},
+	}
+
+	for _, p := range data {
+		values, err := r.ReadAll(p.key)
+		if err != nil {
+			t.Fatalf("unexpected error reading: %v", err)
+		}
+
+		if got, exp := len(values), len(p.points); got != exp {
+			t.Fatalf("values length mismatch %s: got %v, exp %v", p.key, got, exp)
+		}
+
+		for i, point := range p.points {
+			assertValueEqual(t, values[i], point)
+		}
+	}
+}
+
 // Ensures that a compaction will properly merge multiple TSM files
 func TestCompactor_CompactFull_SkipFullBlocks(t *testing.T) {
 	dir := MustTempDir()
@@ -681,7 +830,7 @@ func TestTSMKeyIterator_Single(t *testing.T) {
 
 	r := MustTSMReader(dir, 1, writes)
 
-	iter, err := tsm1.NewTSMKeyIterator(1, false, r)
+	iter, err := tsm1.NewTSMKeyIterator(1, false, nil, r)
 	if err != nil {
 		t.Fatalf("unexpected error creating WALKeyIterator: %v", err)
 	}
@@ -741,7 +890,7 @@ func TestTSMKeyIterator_Duplicate(t *testing.T) {
 
 	r2 := MustTSMReader(dir, 2, writes2)
 
-	iter, err := tsm1.NewTSMKeyIterator(1, false, r1, r2)
+	iter, err := tsm1.NewTSMKeyIterator(1, false, nil, r1, r2)
 	if err != nil {
 		t.Fatalf("unexpected error creating WALKeyIterator: %v", err)
 	}
@@ -802,7 +951,7 @@ func TestTSMKeyIterator_MultipleKeysDeleted(t *testing.T) {
 	r2 := MustTSMReader(dir, 2, points2)
 	r2.Delete([]string{"cpu,host=A#!~#count"})
 
-	iter, err := tsm1.NewTSMKeyIterator(1, false, r1, r2)
+	iter, err := tsm1.NewTSMKeyIterator(1, false, nil, r1, r2)
 	if err != nil {
 		t.Fatalf("unexpected error creating WALKeyIterator: %v", err)
 	}
@@ -844,6 +993,41 @@ func TestTSMKeyIterator_MultipleKeysDeleted(t *testing.T) {
 	}
 }
 
+// Tests that the TSMKeyIterator will abort if the interrupt channel is closed
+func TestTSMKeyIterator_Abort(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+
+	v1 := tsm1.NewValue(1, 1.1)
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{v1},
+	}
+
+	r := MustTSMReader(dir, 1, writes)
+
+	intC := make(chan struct{})
+	iter, err := tsm1.NewTSMKeyIterator(1, false, intC, r)
+	if err != nil {
+		t.Fatalf("unexpected error creating WALKeyIterator: %v", err)
+	}
+
+	var aborted bool
+	for iter.Next() {
+		// Abort
+		close(intC)
+
+		_, _, _, _, err := iter.Read()
+		if err == nil {
+			t.Fatalf("unexpected error read: %v", err)
+		}
+		aborted = err != nil
+	}
+
+	if !aborted {
+		t.Fatalf("iteration not aborted")
+	}
+}
+
 func TestCacheKeyIterator_Single(t *testing.T) {
 	v0 := tsm1.NewValue(1, 1.0)
 
@@ -859,7 +1043,7 @@ func TestCacheKeyIterator_Single(t *testing.T) {
 		}
 	}
 
-	iter := tsm1.NewCacheKeyIterator(c, 1)
+	iter := tsm1.NewCacheKeyIterator(c, 1, nil)
 	var readValues bool
 	for iter.Next() {
 		key, _, _, block, err := iter.Read()
@@ -907,7 +1091,7 @@ func TestCacheKeyIterator_Chunked(t *testing.T) {
 		}
 	}
 
-	iter := tsm1.NewCacheKeyIterator(c, 1)
+	iter := tsm1.NewCacheKeyIterator(c, 1, nil)
 	var readValues bool
 	var chunk int
 	for iter.Next() {
@@ -941,9 +1125,46 @@ func TestCacheKeyIterator_Chunked(t *testing.T) {
 	}
 }
 
+// Tests that the CacheKeyIterator will abort if the interrupt channel is closed
+func TestCacheKeyIterator_Abort(t *testing.T) {
+	v0 := tsm1.NewValue(1, 1.0)
+
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{v0},
+	}
+
+	c := tsm1.NewCache(0, "")
+
+	for k, v := range writes {
+		if err := c.Write(k, v); err != nil {
+			t.Fatalf("failed to write key foo to cache: %s", err.Error())
+		}
+	}
+
+	intC := make(chan struct{})
+
+	iter := tsm1.NewCacheKeyIterator(c, 1, intC)
+
+	var aborted bool
+	for iter.Next() {
+		//Abort
+		close(intC)
+
+		_, _, _, _, err := iter.Read()
+		if err == nil {
+			t.Fatalf("unexpected error read: %v", err)
+		}
+		aborted = err != nil
+	}
+
+	if !aborted {
+		t.Fatalf("iteration not aborted")
+	}
+}
+
 func TestDefaultPlanner_Plan_Min(t *testing.T) {
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return []tsm1.FileStat{
 					tsm1.FileStat{
@@ -960,8 +1181,8 @@ func TestDefaultPlanner_Plan_Min(t *testing.T) {
 					},
 				}
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	tsm := cp.Plan(time.Now())
 	if exp, got := 0, len(tsm); got != exp {
@@ -1003,13 +1224,13 @@ func TestDefaultPlanner_Plan_CombineSequence(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[0], data[1], data[2], data[3]}
 	tsm := cp.Plan(time.Now())
@@ -1065,13 +1286,11 @@ func TestDefaultPlanner_Plan_MultipleGroups(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
-			PathsFn: func() []tsm1.FileStat {
-				return data
-			},
+	cp := tsm1.NewDefaultPlanner(&fakeFileStore{
+		PathsFn: func() []tsm1.FileStat {
+			return data
 		},
-	}
+	}, tsdb.DefaultCompactFullWriteColdDuration)
 
 	expFiles := []tsm1.FileStat{data[0], data[1], data[2], data[3],
 		data[4], data[5], data[6], data[7]}
@@ -1132,13 +1351,13 @@ func TestDefaultPlanner_PlanLevel_SmallestCompactionStep(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[4], data[5]}
 	tsm := cp.PlanLevel(1)
@@ -1164,11 +1383,11 @@ func TestDefaultPlanner_PlanLevel_SplitFile(t *testing.T) {
 			Size: 1 * 1024 * 1024,
 		},
 		tsm1.FileStat{
-			Path: "03-03.tsm1",
+			Path: "03-02.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
 		tsm1.FileStat{
-			Path: "03-04.tsm1",
+			Path: "03-03.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
 		tsm1.FileStat{
@@ -1185,13 +1404,13 @@ func TestDefaultPlanner_PlanLevel_SplitFile(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[0], data[1], data[2], data[3], data[4]}
 	tsm := cp.PlanLevel(3)
@@ -1234,13 +1453,13 @@ func TestDefaultPlanner_PlanLevel_IsolatedLowLevel(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[2], data[3]}
 	tsm := cp.PlanLevel(1)
@@ -1287,13 +1506,13 @@ func TestDefaultPlanner_PlanLevel_IsolatedHighLevel(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{}
 	tsm := cp.PlanLevel(3)
@@ -1330,13 +1549,13 @@ func TestDefaultPlanner_PlanLevel3_MinFiles(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{}
 	tsm := cp.PlanLevel(3)
@@ -1362,13 +1581,13 @@ func TestDefaultPlanner_PlanLevel2_MinFiles(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{}
 	tsm := cp.PlanLevel(2)
@@ -1406,13 +1625,13 @@ func TestDefaultPlanner_PlanLevel_Tombstone(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[0], data[1]}
 	tsm := cp.PlanLevel(3)
@@ -1455,13 +1674,13 @@ func TestDefaultPlanner_PlanLevel_Multiple(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles1 := []tsm1.FileStat{data[0], data[1], data[2], data[3]}
 	expFiles2 := []tsm1.FileStat{data[4], data[5]}
@@ -1504,13 +1723,13 @@ func TestDefaultPlanner_PlanOptimize_NoLevel4(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{}
 	tsm := cp.PlanOptimize()
@@ -1547,13 +1766,13 @@ func TestDefaultPlanner_PlanOptimize_Level4(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles1 := []tsm1.FileStat{data[0], data[1], data[2], data[3]}
 	tsm := cp.PlanOptimize()
@@ -1612,13 +1831,13 @@ func TestDefaultPlanner_PlanOptimize_Multiple(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles1 := []tsm1.FileStat{data[0], data[1], data[2], data[3]}
 	expFiles2 := []tsm1.FileStat{data[5], data[6], data[7], data[8]}
@@ -1665,13 +1884,13 @@ func TestDefaultPlanner_PlanOptimize_Optimized(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{}
 	tsm := cp.PlanOptimize()
@@ -1697,13 +1916,13 @@ func TestDefaultPlanner_PlanOptimize_Tombstones(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[0], data[1], data[2]}
 	tsm := cp.PlanOptimize()
@@ -1749,14 +1968,14 @@ func TestDefaultPlanner_Plan_FullOnCold(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
 		},
-		CompactFullWriteColdDuration: time.Nanosecond,
-	}
+		time.Nanosecond,
+	)
 
 	tsm := cp.Plan(time.Now().Add(-time.Second))
 	if exp, got := len(data), len(tsm[0]); got != exp {
@@ -1784,13 +2003,13 @@ func TestDefaultPlanner_Plan_SkipMaxSizeFiles(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	tsm := cp.Plan(time.Now())
 	if exp, got := 0, len(tsm); got != exp {
@@ -1827,15 +2046,13 @@ func TestDefaultPlanner_Plan_SkipPlanningAfterFull(t *testing.T) {
 		blockCount: 1000,
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore:                    fs,
-		CompactFullWriteColdDuration: time.Nanosecond,
-	}
-
+	cp := tsm1.NewDefaultPlanner(fs, time.Nanosecond)
+	plan := cp.Plan(time.Now().Add(-time.Second))
 	// first verify that our test set would return files
-	if exp, got := 4, len(cp.Plan(time.Now().Add(-time.Second))[0]); got != exp {
+	if exp, got := 4, len(plan[0]); got != exp {
 		t.Fatalf("tsm file length mismatch: got %v, exp %v", got, exp)
 	}
+	cp.Release(plan)
 
 	// skip planning if all files are over the limit
 	over := []tsm1.FileStat{
@@ -1869,14 +2086,18 @@ func TestDefaultPlanner_Plan_SkipPlanningAfterFull(t *testing.T) {
 	}
 
 	cp.FileStore = overFs
-	if exp, got := 0, len(cp.Plan(time.Now().Add(-time.Second))); got != exp {
+	plan = cp.Plan(time.Now().Add(-time.Second))
+	if exp, got := 0, len(plan); got != exp {
 		t.Fatalf("tsm file length mismatch: got %v, exp %v", got, exp)
 	}
+	cp.Release(plan)
 
+	plan = cp.PlanOptimize()
 	// ensure the optimize planner would pick this up
-	if exp, got := 1, len(cp.PlanOptimize()); got != exp {
+	if exp, got := 1, len(plan); got != exp {
 		t.Fatalf("tsm file length mismatch: got %v, exp %v", got, exp)
 	}
+	cp.Release(plan)
 
 	cp.FileStore = fs
 	// ensure that it will plan if last modified has changed
@@ -1934,15 +2155,14 @@ func TestDefaultPlanner_Plan_TwoGenLevel3(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			blockCount: 1000,
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
 		},
-		CompactFullWriteColdDuration: time.Hour,
-	}
+		time.Hour)
 
 	tsm := cp.Plan(time.Now().Add(-24 * time.Hour))
 	if exp, got := 1, len(tsm); got != exp {
@@ -1979,15 +2199,17 @@ func TestDefaultPlanner_Plan_NotFullOverMaxsize(t *testing.T) {
 		blockCount: 100,
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore:                    fs,
-		CompactFullWriteColdDuration: time.Nanosecond,
-	}
+	cp := tsm1.NewDefaultPlanner(
+		fs,
+		time.Nanosecond,
+	)
 
+	plan := cp.Plan(time.Now().Add(-time.Second))
 	// first verify that our test set would return files
-	if exp, got := 4, len(cp.Plan(time.Now().Add(-time.Second))[0]); got != exp {
+	if exp, got := 4, len(plan[0]); got != exp {
 		t.Fatalf("tsm file length mismatch: got %v, exp %v", got, exp)
 	}
+	cp.Release(plan)
 
 	// skip planning if all files are over the limit
 	over := []tsm1.FileStat{
@@ -2040,13 +2262,13 @@ func TestDefaultPlanner_Plan_CompactsMiddleSteps(t *testing.T) {
 		},
 	}
 
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return data
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	expFiles := []tsm1.FileStat{data[0], data[1], data[2], data[3]}
 	tsm := cp.Plan(time.Now())
@@ -2062,8 +2284,8 @@ func TestDefaultPlanner_Plan_CompactsMiddleSteps(t *testing.T) {
 }
 
 func TestDefaultPlanner_Plan_LargeSets(t *testing.T) {
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return []tsm1.FileStat{
 					tsm1.FileStat{
@@ -2088,8 +2310,8 @@ func TestDefaultPlanner_Plan_LargeSets(t *testing.T) {
 					},
 				}
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	tsm := cp.Plan(time.Now())
 	if exp, got := 0, len(tsm); got != exp {
@@ -2098,8 +2320,8 @@ func TestDefaultPlanner_Plan_LargeSets(t *testing.T) {
 }
 
 func TestDefaultPlanner_Plan_LargeGeneration(t *testing.T) {
-	cp := &tsm1.DefaultPlanner{
-		FileStore: &fakeFileStore{
+	cp := tsm1.NewDefaultPlanner(
+		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return []tsm1.FileStat{
 					tsm1.FileStat{
@@ -2124,8 +2346,8 @@ func TestDefaultPlanner_Plan_LargeGeneration(t *testing.T) {
 					},
 				}
 			},
-		},
-	}
+		}, tsdb.DefaultCompactFullWriteColdDuration,
+	)
 
 	tsm := cp.Plan(time.Now())
 	if exp, got := 0, len(tsm); got != exp {
@@ -2142,32 +2364,6 @@ func assertValueEqual(t *testing.T, a, b tsm1.Value) {
 	}
 }
 
-func assertEqual(t *testing.T, a tsm1.Value, b models.Point, field string) {
-	if got, exp := a.UnixNano(), b.UnixNano(); got != exp {
-		t.Fatalf("time mismatch: got %v, exp %v", got, exp)
-	}
-	if got, exp := a.Value(), b.Fields()[field]; got != exp {
-		t.Fatalf("value mismatch: got %v, exp %v", got, exp)
-	}
-}
-
-func MustWALSegment(dir string, entries []tsm1.WALEntry) *tsm1.WALSegmentReader {
-	f := MustTempFile(dir)
-	w := tsm1.NewWALSegmentWriter(f)
-
-	for _, e := range entries {
-		if err := w.Write(mustMarshalEntry(e)); err != nil {
-			panic(fmt.Sprintf("write WAL entry: %v", err))
-		}
-	}
-
-	if _, err := f.Seek(0, os.SEEK_SET); err != nil {
-		panic(fmt.Sprintf("seek WAL: %v", err))
-	}
-
-	return tsm1.NewWALSegmentReader(f)
-}
-
 func MustTSMWriter(dir string, gen int) (tsm1.TSMWriter, string) {
 	f := MustTempFile(dir)
 	oldName := f.Name()
@@ -2234,14 +2430,6 @@ func MustOpenTSMReader(name string) *tsm1.TSMReader {
 	return r
 }
 
-type fakeWAL struct {
-	ClosedSegmentsFn func() ([]string, error)
-}
-
-func (w *fakeWAL) ClosedSegments() ([]string, error) {
-	return w.ClosedSegmentsFn()
-}
-
 type fakeFileStore struct {
 	PathsFn      func() []tsm1.FileStat
 	lastModified time.Time
