@@ -4,23 +4,30 @@ import (
 	"archive/tar"
 	"bytes"
 	"fmt"
+	"io"
 	"io/ioutil"
+	"math"
 	"math/rand"
 	"os"
 	"path/filepath"
 	"reflect"
 	"runtime"
 	"strings"
+	"sync"
 	"testing"
 	"time"
 
+	"path"
+
 	"github.com/influxdata/influxdb/influxql"
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/pkg/deep"
 	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/tsdb/index/inmem"
 )
 
+/*
 // Ensure engine can load the metadata index after reopening.
 func TestEngine_LoadMetadataIndex(t *testing.T) {
 	e := MustOpenEngine()
@@ -36,16 +43,19 @@ func TestEngine_LoadMetadataIndex(t *testing.T) {
 	}
 
 	// Load metadata index.
-	index := tsdb.NewDatabaseIndex("db")
+	index := MustNewDatabaseIndex("db")
 	if err := e.LoadMetadataIndex(1, index); err != nil {
 		t.Fatal(err)
 	}
 
 	// Verify index is correct.
-	if m := index.Measurement("cpu"); m == nil {
+	m, err := index.Measurement([]byte("cpu"))
+	if err != nil {
+		t.Fatal(err)
+	} else if m == nil {
 		t.Fatal("measurement not found")
-	} else if s := m.SeriesByID(1); s.Key != "cpu,host=A" || !reflect.DeepEqual(s.Tags, models.NewTags(map[string]string{"host": "A"})) {
-		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags)
+	} else if s := m.SeriesByID(1); s.Key != "cpu,host=A" || !reflect.DeepEqual(s.Tags(), models.NewTags(map[string]string{"host": "A"})) {
+		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags())
 	}
 
 	// write the snapshot, ensure we can close and load index from TSM
@@ -59,16 +69,18 @@ func TestEngine_LoadMetadataIndex(t *testing.T) {
 	}
 
 	// Load metadata index.
-	index = tsdb.NewDatabaseIndex("db")
+	index = MustNewDatabaseIndex("db")
 	if err := e.LoadMetadataIndex(1, index); err != nil {
 		t.Fatal(err)
 	}
 
 	// Verify index is correct.
-	if m := index.Measurement("cpu"); m == nil {
+	if m, err = index.Measurement([]byte("cpu")); err != nil {
+		t.Fatal(err)
+	} else if m == nil {
 		t.Fatal("measurement not found")
-	} else if s := m.SeriesByID(1); s.Key != "cpu,host=A" || !reflect.DeepEqual(s.Tags, models.NewTags(map[string]string{"host": "A"})) {
-		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags)
+	} else if s := m.SeriesByID(1); s.Key != "cpu,host=A" || !reflect.DeepEqual(s.Tags(), models.NewTags(map[string]string{"host": "A"})) {
+		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags())
 	}
 
 	// Write a new point and ensure we can close and load index from TSM and WAL
@@ -84,20 +96,23 @@ func TestEngine_LoadMetadataIndex(t *testing.T) {
 	}
 
 	// Load metadata index.
-	index = tsdb.NewDatabaseIndex("db")
+	index = MustNewDatabaseIndex("db")
 	if err := e.LoadMetadataIndex(1, index); err != nil {
 		t.Fatal(err)
 	}
 
 	// Verify index is correct.
-	if m := index.Measurement("cpu"); m == nil {
+	if m, err = index.Measurement([]byte("cpu")); err != nil {
+		t.Fatal(err)
+	} else if m == nil {
 		t.Fatal("measurement not found")
-	} else if s := m.SeriesByID(1); s.Key != "cpu,host=A" || !reflect.DeepEqual(s.Tags, models.NewTags(map[string]string{"host": "A"})) {
-		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags)
-	} else if s := m.SeriesByID(2); s.Key != "cpu,host=B" || !reflect.DeepEqual(s.Tags, models.NewTags(map[string]string{"host": "B"})) {
-		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags)
+	} else if s := m.SeriesByID(1); s.Key != "cpu,host=A" || !reflect.DeepEqual(s.Tags(), models.NewTags(map[string]string{"host": "A"})) {
+		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags())
+	} else if s := m.SeriesByID(2); s.Key != "cpu,host=B" || !reflect.DeepEqual(s.Tags(), models.NewTags(map[string]string{"host": "B"})) {
+		t.Fatalf("unexpected series: %q / %#v", s.Key, s.Tags())
 	}
 }
+*/
 
 // Ensure that deletes only sent to the WAL will clear out the data from the cache on restart
 func TestEngine_DeleteWALLoadMetadata(t *testing.T) {
@@ -112,7 +127,7 @@ func TestEngine_DeleteWALLoadMetadata(t *testing.T) {
 	}
 
 	// Remove series.
-	if err := e.DeleteSeries([]string{"cpu,host=A"}); err != nil {
+	if err := e.DeleteSeriesRange([][]byte{[]byte("cpu,host=A")}, math.MinInt64, math.MaxInt64); err != nil {
 		t.Fatalf("failed to delete series: %s", err.Error())
 	}
 
@@ -146,7 +161,13 @@ func TestEngine_Backup(t *testing.T) {
 	p3 := MustParsePointString("cpu,host=C value=1.3 3000000000")
 
 	// Write those points to the engine.
-	e := tsm1.NewEngine(1, f.Name(), walPath, tsdb.NewEngineOptions()).(*tsm1.Engine)
+	db := path.Base(f.Name())
+	opt := tsdb.NewEngineOptions()
+	opt.InmemIndex = inmem.NewIndex(db)
+	idx := tsdb.MustOpenIndex(1, db, filepath.Join(f.Name(), "index"), opt)
+	defer idx.Close()
+
+	e := tsm1.NewEngine(1, idx, db, f.Name(), walPath, opt).(*tsm1.Engine)
 
 	// mock the planner so compactions don't run during the test
 	e.CompactionPlan = &mockPlanner{}
@@ -176,14 +197,30 @@ func TestEngine_Backup(t *testing.T) {
 		t.Fatalf("file count wrong: exp: %d, got: %d", 2, len(e.FileStore.Files()))
 	}
 
+	fileNames := map[string]bool{}
 	for _, f := range e.FileStore.Files() {
-		th, err := tr.Next()
-		if err != nil {
-			t.Fatalf("failed reading header: %s", err.Error())
-		}
-		if !strings.Contains(f.Path(), th.Name) || th.Name == "" {
-			t.Fatalf("file name doesn't match:\n\tgot: %s\n\texp: %s", th.Name, f.Path())
+		fileNames[filepath.Base(f.Path())] = true
+	}
+
+	th, err := tr.Next()
+	for err == nil {
+		if !fileNames[th.Name] {
+			t.Errorf("Extra file in backup: %q", th.Name)
 		}
+		delete(fileNames, th.Name)
+		th, err = tr.Next()
+	}
+
+	if err != nil && err != io.EOF {
+		t.Fatalf("Problem reading tar header: %s", err)
+	}
+
+	for f := range fileNames {
+		t.Errorf("File missing from backup: %s", f)
+	}
+
+	if t.Failed() {
+		t.FailNow()
 	}
 
 	lastBackup := time.Now()
@@ -202,7 +239,7 @@ func TestEngine_Backup(t *testing.T) {
 	}
 
 	tr = tar.NewReader(b)
-	th, err := tr.Next()
+	th, err = tr.Next()
 	if err != nil {
 		t.Fatalf("error getting next tar header: %s", err.Error())
 	}
@@ -220,10 +257,9 @@ func TestEngine_CreateIterator_Cache_Ascending(t *testing.T) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	// e.CreateMeasurement("cpu")
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
 
 	if err := e.WritePointsString(
 		`cpu,host=A value=1.1 1000000000`,
@@ -233,10 +269,9 @@ func TestEngine_CreateIterator_Cache_Ascending(t *testing.T) {
 		t.Fatalf("failed to write points: %s", err.Error())
 	}
 
-	itr, err := e.CreateIterator(influxql.IteratorOptions{
+	itr, err := e.CreateIterator("cpu", influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
 		EndTime:    influxql.MaxTime,
 		Ascending:  true,
@@ -275,10 +310,8 @@ func TestEngine_CreateIterator_Cache_Descending(t *testing.T) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
 
 	if err := e.WritePointsString(
 		`cpu,host=A value=1.1 1000000000`,
@@ -288,10 +321,9 @@ func TestEngine_CreateIterator_Cache_Descending(t *testing.T) {
 		t.Fatalf("failed to write points: %s", err.Error())
 	}
 
-	itr, err := e.CreateIterator(influxql.IteratorOptions{
+	itr, err := e.CreateIterator("cpu", influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
 		EndTime:    influxql.MaxTime,
 		Ascending:  false,
@@ -330,10 +362,8 @@ func TestEngine_CreateIterator_TSM_Ascending(t *testing.T) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
 
 	if err := e.WritePointsString(
 		`cpu,host=A value=1.1 1000000000`,
@@ -344,10 +374,9 @@ func TestEngine_CreateIterator_TSM_Ascending(t *testing.T) {
 	}
 	e.MustWriteSnapshot()
 
-	itr, err := e.CreateIterator(influxql.IteratorOptions{
+	itr, err := e.CreateIterator("cpu", influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
 		EndTime:    influxql.MaxTime,
 		Ascending:  true,
@@ -386,10 +415,8 @@ func TestEngine_CreateIterator_TSM_Descending(t *testing.T) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
 
 	if err := e.WritePointsString(
 		`cpu,host=A value=1.1 1000000000`,
@@ -400,10 +427,9 @@ func TestEngine_CreateIterator_TSM_Descending(t *testing.T) {
 	}
 	e.MustWriteSnapshot()
 
-	itr, err := e.CreateIterator(influxql.IteratorOptions{
+	itr, err := e.CreateIterator("cpu", influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
 		EndTime:    influxql.MaxTime,
 		Ascending:  false,
@@ -442,11 +468,9 @@ func TestEngine_CreateIterator_Aux(t *testing.T) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("F", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("F"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
 
 	if err := e.WritePointsString(
 		`cpu,host=A value=1.1 1000000000`,
@@ -458,11 +482,10 @@ func TestEngine_CreateIterator_Aux(t *testing.T) {
 		t.Fatalf("failed to write points: %s", err.Error())
 	}
 
-	itr, err := e.CreateIterator(influxql.IteratorOptions{
+	itr, err := e.CreateIterator("cpu", influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
 		Aux:        []influxql.VarRef{{Val: "F"}},
 		Dimensions: []string{"host"},
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
 		EndTime:    influxql.MaxTime,
 		Ascending:  true,
@@ -501,14 +524,12 @@ func TestEngine_CreateIterator_Condition(t *testing.T) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.Index().Measurement("cpu").SetFieldName("X")
-	e.Index().Measurement("cpu").SetFieldName("Y")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("X", influxql.Float, false)
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("Y", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("X"), influxql.Float, false)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("Y"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
+	e.SetFieldName([]byte("cpu"), "X")
+	e.SetFieldName([]byte("cpu"), "Y")
 
 	if err := e.WritePointsString(
 		`cpu,host=A value=1.1 1000000000`,
@@ -524,11 +545,10 @@ func TestEngine_CreateIterator_Condition(t *testing.T) {
 		t.Fatalf("failed to write points: %s", err.Error())
 	}
 
-	itr, err := e.CreateIterator(influxql.IteratorOptions{
+	itr, err := e.CreateIterator("cpu", influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr(`value`),
 		Dimensions: []string{"host"},
 		Condition:  influxql.MustParseExpr(`X = 10 OR Y > 150`),
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		StartTime:  influxql.MinTime,
 		EndTime:    influxql.MaxTime,
 		Ascending:  true,
@@ -572,7 +592,13 @@ func TestEngine_DeleteSeries(t *testing.T) {
 	p3 := MustParsePointString("cpu,host=A sum=1.3 3000000000")
 
 	// Write those points to the engine.
-	e := tsm1.NewEngine(1, f.Name(), walPath, tsdb.NewEngineOptions()).(*tsm1.Engine)
+	db := path.Base(f.Name())
+	opt := tsdb.NewEngineOptions()
+	opt.InmemIndex = inmem.NewIndex(db)
+	idx := tsdb.MustOpenIndex(1, db, filepath.Join(f.Name(), "index"), opt)
+	defer idx.Close()
+	e := tsm1.NewEngine(1, idx, db, f.Name(), walPath, opt).(*tsm1.Engine)
+	// e.LoadMetadataIndex(1, MustNewDatabaseIndex("db0")) // Initialise an index
 
 	// mock the planner so compactions don't run during the test
 	e.CompactionPlan = &mockPlanner{}
@@ -593,7 +619,7 @@ func TestEngine_DeleteSeries(t *testing.T) {
 		t.Fatalf("series count mismatch: exp %v, got %v", exp, got)
 	}
 
-	if err := e.DeleteSeries([]string{"cpu,host=A"}); err != nil {
+	if err := e.DeleteSeriesRange([][]byte{[]byte("cpu,host=A")}, math.MinInt64, math.MaxInt64); err != nil {
 		t.Fatalf("failed to delete series: %v", err)
 	}
 
@@ -609,6 +635,104 @@ func TestEngine_DeleteSeries(t *testing.T) {
 
 }
 
+func TestEngine_LastModified(t *testing.T) {
+	// Generate temporary file.
+	dir, _ := ioutil.TempDir("", "tsm")
+	walPath := filepath.Join(dir, "wal")
+	os.MkdirAll(walPath, 0777)
+	defer os.RemoveAll(dir)
+
+	// Create a few points.
+	p1 := MustParsePointString("cpu,host=A value=1.1 1000000000")
+	p2 := MustParsePointString("cpu,host=B value=1.2 2000000000")
+	p3 := MustParsePointString("cpu,host=A sum=1.3 3000000000")
+
+	// Write those points to the engine.
+	db := path.Base(dir)
+	opt := tsdb.NewEngineOptions()
+	opt.InmemIndex = inmem.NewIndex(db)
+	idx := tsdb.MustOpenIndex(1, db, filepath.Join(dir, "index"), opt)
+	defer idx.Close()
+
+	e := tsm1.NewEngine(1, idx, db, dir, walPath, opt).(*tsm1.Engine)
+
+	// mock the planner so compactions don't run during the test
+	e.CompactionPlan = &mockPlanner{}
+
+	if lm := e.LastModified(); !lm.IsZero() {
+		t.Fatalf("expected zero time, got %v", lm.UTC())
+	}
+
+	e.SetEnabled(false)
+	if err := e.Open(); err != nil {
+		t.Fatalf("failed to open tsm1 engine: %s", err.Error())
+	}
+
+	if err := e.WritePoints([]models.Point{p1, p2, p3}); err != nil {
+		t.Fatalf("failed to write points: %s", err.Error())
+	}
+
+	lm := e.LastModified()
+	if lm.IsZero() {
+		t.Fatalf("expected non-zero time, got %v", lm.UTC())
+	}
+	e.SetEnabled(true)
+
+	if err := e.WriteSnapshot(); err != nil {
+		t.Fatalf("failed to snapshot: %s", err.Error())
+	}
+
+	lm2 := e.LastModified()
+
+	if got, exp := lm.Equal(lm2), false; exp != got {
+		t.Fatalf("expected time change, got %v, exp %v", got, exp)
+	}
+
+	if err := e.DeleteSeriesRange([][]byte{[]byte("cpu,host=A")}, math.MinInt64, math.MaxInt64); err != nil {
+		t.Fatalf("failed to delete series: %v", err)
+	}
+
+	lm3 := e.LastModified()
+	if got, exp := lm2.Equal(lm3), false; exp != got {
+		t.Fatalf("expected time change, got %v, exp %v", got, exp)
+	}
+}
+
+func TestEngine_SnapshotsDisabled(t *testing.T) {
+	// Generate temporary file.
+	dir, _ := ioutil.TempDir("", "tsm")
+	walPath := filepath.Join(dir, "wal")
+	os.MkdirAll(walPath, 0777)
+	defer os.RemoveAll(dir)
+
+	// Create a tsm1 engine.
+	db := path.Base(dir)
+	opt := tsdb.NewEngineOptions()
+	opt.InmemIndex = inmem.NewIndex(db)
+	idx := tsdb.MustOpenIndex(1, db, filepath.Join(dir, "index"), opt)
+	defer idx.Close()
+
+	e := tsm1.NewEngine(1, idx, db, dir, walPath, opt).(*tsm1.Engine)
+
+	// mock the planner so compactions don't run during the test
+	e.CompactionPlan = &mockPlanner{}
+
+	e.SetEnabled(false)
+	if err := e.Open(); err != nil {
+		t.Fatalf("failed to open tsm1 engine: %s", err.Error())
+	}
+
+	// Make sure Snapshots are disabled.
+	e.SetCompactionsEnabled(false)
+	e.Compactor.DisableSnapshots()
+
+	// Writing a snapshot should not fail when the snapshot is empty
+	// even if snapshots are disabled.
+	if err := e.WriteSnapshot(); err != nil {
+		t.Fatalf("failed to snapshot: %s", err.Error())
+	}
+}
+
 func BenchmarkEngine_CreateIterator_Count_1K(b *testing.B) {
 	benchmarkEngineCreateIteratorCount(b, 1000)
 }
@@ -622,7 +746,6 @@ func BenchmarkEngine_CreateIterator_Count_1M(b *testing.B) {
 func benchmarkEngineCreateIteratorCount(b *testing.B, pointN int) {
 	benchmarkIterator(b, influxql.IteratorOptions{
 		Expr:      influxql.MustParseExpr("count(value)"),
-		Sources:   []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		Ascending: true,
 		StartTime: influxql.MinTime,
 		EndTime:   influxql.MaxTime,
@@ -642,7 +765,6 @@ func BenchmarkEngine_CreateIterator_First_1M(b *testing.B) {
 func benchmarkEngineCreateIteratorFirst(b *testing.B, pointN int) {
 	benchmarkIterator(b, influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr("first(value)"),
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		Dimensions: []string{"host"},
 		Ascending:  true,
 		StartTime:  influxql.MinTime,
@@ -663,7 +785,6 @@ func BenchmarkEngine_CreateIterator_Last_1M(b *testing.B) {
 func benchmarkEngineCreateIteratorLast(b *testing.B, pointN int) {
 	benchmarkIterator(b, influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr("last(value)"),
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		Dimensions: []string{"host"},
 		Ascending:  true,
 		StartTime:  influxql.MinTime,
@@ -684,9 +805,11 @@ func BenchmarkEngine_CreateIterator_Limit_1M(b *testing.B) {
 func BenchmarkEngine_WritePoints_10(b *testing.B) {
 	benchmarkEngine_WritePoints(b, 10)
 }
+
 func BenchmarkEngine_WritePoints_100(b *testing.B) {
 	benchmarkEngine_WritePoints(b, 100)
 }
+
 func BenchmarkEngine_WritePoints_1000(b *testing.B) {
 	benchmarkEngine_WritePoints(b, 1000)
 }
@@ -699,8 +822,7 @@ func benchmarkEngine_WritePoints(b *testing.B, batchSize int) {
 	e := MustOpenEngine()
 	defer e.Close()
 
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
 
 	pp := make([]models.Point, 0, batchSize)
 	for i := 0; i < batchSize; i++ {
@@ -718,10 +840,89 @@ func benchmarkEngine_WritePoints(b *testing.B, batchSize int) {
 	}
 }
 
+func BenchmarkEngine_WritePoints_Parallel_1000(b *testing.B) {
+	benchmarkEngine_WritePoints(b, 1000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_5000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 5000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_10000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 10000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_25000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 25000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_50000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 50000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_75000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 75000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_100000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 100000)
+}
+
+func BenchmarkEngine_WritePoints_Parallel_200000(b *testing.B) {
+	benchmarkEngine_WritePoints_Parallel(b, 200000)
+}
+
+func benchmarkEngine_WritePoints_Parallel(b *testing.B, batchSize int) {
+	e := MustOpenEngine()
+	defer e.Close()
+
+	// e.Index().CreateMeasurementIndexIfNotExists("cpu")
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+
+	b.ResetTimer()
+	b.ReportAllocs()
+	for i := 0; i < b.N; i++ {
+
+		b.StopTimer()
+		cpus := runtime.GOMAXPROCS(0)
+		pp := make([]models.Point, 0, batchSize*cpus)
+		for i := 0; i < batchSize*cpus; i++ {
+			p := MustParsePointString(fmt.Sprintf("cpu,host=%d value=1.2,other=%di", i, i))
+			pp = append(pp, p)
+		}
+		b.StartTimer()
+
+		var wg sync.WaitGroup
+		errC := make(chan error)
+		for i := 0; i < cpus; i++ {
+			wg.Add(1)
+			go func(i int) {
+				defer wg.Done()
+				from, to := i*batchSize, (i+1)*batchSize
+				err := e.WritePoints(pp[from:to])
+				if err != nil {
+					errC <- err
+					return
+				}
+			}(i)
+		}
+
+		go func() {
+			wg.Wait()
+			close(errC)
+		}()
+
+		for err := range errC {
+			if err != nil {
+				b.Error(err)
+			}
+		}
+	}
+}
+
 func benchmarkEngineCreateIteratorLimit(b *testing.B, pointN int) {
 	benchmarkIterator(b, influxql.IteratorOptions{
 		Expr:       influxql.MustParseExpr("value"),
-		Sources:    []influxql.Source{&influxql.Measurement{Name: "cpu"}},
 		Dimensions: []string{"host"},
 		Ascending:  true,
 		StartTime:  influxql.MinTime,
@@ -736,7 +937,7 @@ func benchmarkIterator(b *testing.B, opt influxql.IteratorOptions, pointN int) {
 	b.ReportAllocs()
 
 	for i := 0; i < b.N; i++ {
-		itr, err := e.CreateIterator(opt)
+		itr, err := e.CreateIterator("cpu", opt)
 		if err != nil {
 			b.Fatal(err)
 		}
@@ -773,10 +974,8 @@ func MustInitBenchmarkEngine(pointN int) *Engine {
 	e := MustOpenEngine()
 
 	// Initialize metadata.
-	e.Index().CreateMeasurementIndexIfNotExists("cpu")
-	e.MeasurementFields("cpu").CreateFieldIfNotExists("value", influxql.Float, false)
-	si := e.Index().CreateSeriesIndexIfNotExists("cpu", tsdb.NewSeries("cpu,host=A", models.NewTags(map[string]string{"host": "A"})))
-	si.AssignShard(1)
+	e.MeasurementFields([]byte("cpu")).CreateFieldIfNotExists([]byte("value"), influxql.Float, false)
+	e.CreateSeriesIfNotExists([]byte("cpu,host=A"), []byte("cpu"), models.NewTags(map[string]string{"host": "A"}))
 
 	// Generate time ascending points with jitterred time & value.
 	rand := rand.New(rand.NewSource(0))
@@ -815,7 +1014,8 @@ func MustInitBenchmarkEngine(pointN int) *Engine {
 // Engine is a test wrapper for tsm1.Engine.
 type Engine struct {
 	*tsm1.Engine
-	root string
+	root  string
+	index tsdb.Index
 }
 
 // NewEngine returns a new instance of Engine at a temporary location.
@@ -824,12 +1024,22 @@ func NewEngine() *Engine {
 	if err != nil {
 		panic(err)
 	}
+
+	db := path.Base(root)
+	opt := tsdb.NewEngineOptions()
+	opt.InmemIndex = inmem.NewIndex(db)
+
+	idx := tsdb.MustOpenIndex(1, db, filepath.Join(root, "data", "index"), opt)
+
 	return &Engine{
 		Engine: tsm1.NewEngine(1,
+			idx,
+			db,
 			filepath.Join(root, "data"),
 			filepath.Join(root, "wal"),
-			tsdb.NewEngineOptions()).(*tsm1.Engine),
-		root: root,
+			opt).(*tsm1.Engine),
+		root:  root,
+		index: idx,
 	}
 }
 
@@ -839,14 +1049,17 @@ func MustOpenEngine() *Engine {
 	if err := e.Open(); err != nil {
 		panic(err)
 	}
-	if err := e.LoadMetadataIndex(1, tsdb.NewDatabaseIndex("db")); err != nil {
-		panic(err)
-	}
+	// if err := e.LoadMetadataIndex(1, MustNewDatabaseIndex("db")); err != nil {
+	// 	panic(err)
+	// }
 	return e
 }
 
 // Close closes the engine and removes all underlying data.
 func (e *Engine) Close() error {
+	if e.index != nil {
+		e.index.Close()
+	}
 	defer os.RemoveAll(e.root)
 	return e.Engine.Close()
 }
@@ -855,12 +1068,22 @@ func (e *Engine) Close() error {
 func (e *Engine) Reopen() error {
 	if err := e.Engine.Close(); err != nil {
 		return err
+	} else if e.index.Close(); err != nil {
+		return err
 	}
 
+	db := path.Base(e.root)
+	opt := tsdb.NewEngineOptions()
+	opt.InmemIndex = inmem.NewIndex(db)
+
+	e.index = tsdb.MustOpenIndex(1, db, filepath.Join(e.root, "data", "index"), opt)
+
 	e.Engine = tsm1.NewEngine(1,
+		e.index,
+		db,
 		filepath.Join(e.root, "data"),
 		filepath.Join(e.root, "wal"),
-		tsdb.NewEngineOptions()).(*tsm1.Engine)
+		opt).(*tsm1.Engine)
 
 	if err := e.Engine.Open(); err != nil {
 		return err
@@ -897,6 +1120,8 @@ type mockPlanner struct{}
 func (m *mockPlanner) Plan(lastWrite time.Time) []tsm1.CompactionGroup { return nil }
 func (m *mockPlanner) PlanLevel(level int) []tsm1.CompactionGroup      { return nil }
 func (m *mockPlanner) PlanOptimize() []tsm1.CompactionGroup            { return nil }
+func (m *mockPlanner) Release(groups []tsm1.CompactionGroup)           {}
+func (m *mockPlanner) FullyCompacted() bool                            { return false }
 
 // ParseTags returns an instance of Tags for a comma-delimited list of key/values.
 func ParseTags(s string) influxql.Tags {
